@@ -26,11 +26,13 @@ from neutron_lib import exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import netutils
-from pyroute2.netlink import exceptions as netlink_exceptions
-from pyroute2.netlink import rtnl
-from pyroute2.netlink.rtnl import ifaddrmsg
-from pyroute2.netlink.rtnl import ifinfmsg
-from pyroute2 import netns
+from pyroute2.netlink import exceptions \
+    as netlink_exceptions  # pylint: disable=no-name-in-module
+from pyroute2.netlink import rtnl  # pylint: disable=no-name-in-module
+from pyroute2.netlink.rtnl import \
+    ifaddrmsg  # pylint: disable=no-name-in-module
+from pyroute2.netlink.rtnl import ifinfmsg  # pylint: disable=no-name-in-module
+from pyroute2 import netns  # pylint: disable=no-name-in-module
 
 from neutron._i18n import _
 from neutron.agent.common import utils
@@ -276,11 +278,20 @@ class IPWrapper(SubProcessBase):
                                     vlan_id=vlan_id)
         return IPDevice(name, namespace=self.namespace)
 
-    def add_vxlan(self, name, vni, group=None, dev=None, ttl=None, tos=None,
+    def add_vxlan(self, name, vni, dev, group=None, ttl=None, tos=None,
                   local=None, srcport=None, dstport=None, proxy=False):
-        kwargs = {'vxlan_id': vni}
+        kwargs = {'vxlan_id': vni,
+                  'physical_interface': dev}
         if group:
-            kwargs['vxlan_group'] = group
+            try:
+                ip_version = common_utils.get_ip_version(group)
+                if ip_version == constants.IP_VERSION_6:
+                    kwargs['vxlan_group6'] = group
+                else:
+                    kwargs['vxlan_group'] = group
+            except netaddr.core.AddrFormatError:
+                err_msg = _("Invalid group address: %s") % group
+                raise exceptions.InvalidInput(error_message=err_msg)
         if dev:
             kwargs['physical_interface'] = dev
         if ttl:
@@ -288,7 +299,15 @@ class IPWrapper(SubProcessBase):
         if tos:
             kwargs['vxlan_tos'] = tos
         if local:
-            kwargs['vxlan_local'] = local
+            try:
+                ip_version = common_utils.get_ip_version(local)
+                if ip_version == constants.IP_VERSION_6:
+                    kwargs['vxlan_local6'] = local
+                else:
+                    kwargs['vxlan_local'] = local
+            except netaddr.core.AddrFormatError:
+                err_msg = _("Invalid local address: %s") % local
+                raise exceptions.InvalidInput(error_message=err_msg)
         if proxy:
             kwargs['vxlan_proxy'] = proxy
         # tuple: min,max
