@@ -16,6 +16,8 @@
 import collections
 import contextlib
 import copy
+import itertools
+import re
 
 import eventlet
 import netaddr
@@ -41,6 +43,7 @@ from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
         as ovs_consts
 
 LOG = logging.getLogger(__name__)
+CONJ_ID_REGEX = re.compile(r"conj_id=(\d+),")
 
 
 def _replace_register(flow_params, register_number, register_value):
@@ -262,33 +265,75 @@ class SGPortMap(object):
 class ConjIdMap(object):
     """Handle conjunction ID allocations and deallocations."""
 
-    def __new__(cls):
+    CONJ_ID_BLOCK_SIZE = 8
+    MAX_CONJ_ID = 2 ** 32 - 8
+
+    def __new__(cls, int_br):
         if not hasattr(cls, '_instance'):
             cls._instance = super(ConjIdMap, cls).__new__(cls)
         return cls._instance
 
-    def __init__(self):
+    def __init__(self, int_br):
         self.id_map = collections.defaultdict(self._conj_id_factory)
         # Stores the set of conjuntion IDs used for each unique tuple
         # (sg_id, remote_id, direction, ethertype). Each tuple
         # can have up to 8 conjuntion IDs (see ConjIPFlowManager.add()).
         self.id_map_group = collections.defaultdict(set)
         self.id_free = collections.deque()
-        self.max_id = 0
+        self._max_id = self._init_max_id(int_br)
+
+    def _init_max_id(self, int_br):
+        """Read the maximum conjunction ID number in the integration bridge
+
+        This method will dump all integration bridge flows, parse them and
+        return the maximum conjunction ID number. By default, "int_br" is a
+        ``OVSAgentBridge`` instance, using "os-ken" library to access to the OF
+        rules.
+        If not, "int_br" will default to a ``OVSBridge`` instance. The CLI
+        command "ovs-ofctl" will be used instead.
+
+        :param int_br: ``OVSAgentBridge`` or ``OVSBridge`` instance.
+        :returns: The maximum conjunction ID number in the integration bridge
+        """
+        conj_id_max = 0
+        try:
+            for flow in itertools.chain(
+                    *[int_br.dump_flows(table)
+                      for table in ovs_consts.OVS_FIREWALL_TABLES]):
+                conj_id_max = max(conj_id_max, flow.match.get('conj_id', 0))
+        except AttributeError:  # br_int is a ``OVSBridge`` instance.
+            flows_iter = itertools.chain(
+                *[int_br.dump_flows_for_table(table)
+                  for table in ovs_consts.OVS_FIREWALL_TABLES])
+            conj_ids = CONJ_ID_REGEX.findall(" | ".join(flows_iter))
+            try:
+                conj_id_max = max([int(conj_id) for conj_id in conj_ids])
+            except ValueError:
+                conj_id_max = 0
+
+        max_id = conj_id_max - conj_id_max % self.CONJ_ID_BLOCK_SIZE
+        return self._next_max_id(max_id)
+
+    def _next_max_id(self, max_id):
+        max_id += self.CONJ_ID_BLOCK_SIZE
+        if max_id >= self.MAX_CONJ_ID:
+            max_id = 0
+        return max_id
 
     def _conj_id_factory(self):
         # If there is any freed ID, use one.
         if self.id_free:
             return self.id_free.popleft()
         # Allocate new one. It must be divisible by 8. (See the next function.)
-        self.max_id += 8
-        return self.max_id
+        self._max_id = self._next_max_id(self._max_id)
+        return self._max_id
 
     def get_conj_id(self, sg_id, remote_id, direction, ethertype):
         """Return a conjunction ID specified by the arguments.
-        Allocate one if necessary.  The returned ID is divisible by 8,
-        as there are 4 priority levels (see rules.flow_priority_offset)
-        and 2 conjunction IDs are needed per priority.
+        Allocate one if necessary.  The returned ID is divisible by 8
+        (CONJ_ID_BLOCK_SIZE), as there are 4 priority levels
+        (see rules.flow_priority_offset) and 2 conjunction IDs are needed per
+        priority.
         """
         if direction not in [lib_const.EGRESS_DIRECTION,
                              lib_const.INGRESS_DIRECTION]:
@@ -337,7 +382,7 @@ class ConjIPFlowManager(object):
     """
 
     def __init__(self, driver):
-        self.conj_id_map = ConjIdMap()
+        self.conj_id_map = ConjIdMap(driver.int_br.br)
         self.driver = driver
         # The following two are dict of dicts and are indexed like:
         #     self.x[vlan_tag][(direction, ethertype)]
@@ -570,14 +615,6 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         else:
             self.int_br.br.delete_flows(**kwargs)
 
-    def _strict_delete_flow(self, **kwargs):
-        """Delete given flow right away even if bridge is deferred.
-
-        Delete command will use strict delete.
-        """
-        create_reg_numbers(kwargs)
-        self.int_br.br.delete_flows(strict=True, **kwargs)
-
     @staticmethod
     def initialize_bridge(int_br):
         int_br.add_protocols(*OVSFirewallDriver.REQUIRED_PROTOCOLS)
@@ -759,13 +796,6 @@ class OVSFirewallDriver(firewall.FirewallDriver):
     def _update_flows_for_port(self, of_port, old_of_port):
         with self.update_cookie_context():
             self._set_port_filters(of_port)
-        # Flush the flows caused by changes made to deferred bridge. The reason
-        # is that following delete_all_port_flows() call uses --strict
-        # parameter that cannot be combined with other non-strict rules, hence
-        # all parameters with --strict are applied right away. In order to
-        # avoid applying delete rules with --strict *before*
-        # _set_port_filters() we dump currently cached flows here.
-        self.int_br.apply_flows()
         self.delete_all_port_flows(old_of_port)
         # Rewrite update cookie with default cookie
         self._set_port_filters(of_port)
@@ -878,15 +908,13 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
     def delete_physical_direct_flow(self, mac, segment_id):
         if segment_id:
-            self._strict_delete_flow(priority=90,
-                                     table=ovs_consts.TRANSIENT_TABLE,
-                                     dl_dst=mac,
-                                     dl_vlan=segment_id)
+            self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
+                               dl_dst=mac,
+                               dl_vlan=segment_id)
         else:
-            self._strict_delete_flow(priority=90,
-                                     table=ovs_consts.TRANSIENT_TABLE,
-                                     dl_dst=mac,
-                                     vlan_tci=ovs_consts.FLAT_VLAN_TCI)
+            self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
+                               dl_dst=mac,
+                               vlan_tci=ovs_consts.FLAT_VLAN_TCI)
 
     def initialize_port_flows(self, port):
         """Set base flows for port
@@ -934,8 +962,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         self._initialize_ingress(port)
 
     def _initialize_egress_ipv6_icmp(self, port, allowed_pairs):
-        # NOTE(slaweq): should we include also fe80::/64 (link-local) subnet
-        # in the allowed pairs here?
+        allowed_pairs = allowed_pairs.union({(port.mac, port.lla_address)})
         for mac_addr, ip_addr in allowed_pairs:
             for icmp_type in firewall.ICMPV6_ALLOWED_EGRESS_TYPES:
                 self._add_flow(
@@ -951,6 +978,19 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                     actions='resubmit(,%d)' % (
                         ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
                 )
+            for icmp_type in firewall.ICMPV6_RESTRICTED_EGRESS_TYPES:
+                self._add_flow(
+                    table=ovs_consts.BASE_EGRESS_TABLE,
+                    priority=95,
+                    in_port=port.ofport,
+                    reg_port=port.ofport,
+                    dl_type=lib_const.ETHERTYPE_IPV6,
+                    nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
+                    icmp_type=icmp_type,
+                    nd_target=ip_addr,
+                    actions='resubmit(,%d)' % (
+                        ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
+                )
 
     def _initialize_egress_no_port_security(self, port_id, ovs_ports=None):
         try:
@@ -1025,9 +1065,9 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         """Identify egress traffic and send it to egress base"""
 
         # Apply mac/ip pairs for IPv4
-        allowed_pairs = port.allowed_pairs_v4.union(
+        allowed_mac_ipv4_pairs = port.allowed_pairs_v4.union(
             {(port.mac, ip_addr) for ip_addr in port.ipv4_addresses})
-        for mac_addr, ip_addr in allowed_pairs:
+        for mac_addr, ip_addr in allowed_mac_ipv4_pairs:
             self._add_flow(
                 table=ovs_consts.BASE_EGRESS_TABLE,
                 priority=95,
@@ -1053,10 +1093,10 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             )
 
         # Apply mac/ip pairs for IPv6
-        allowed_pairs = port.allowed_pairs_v6.union(
+        allowed_mac_ipv6_pairs = port.allowed_pairs_v6.union(
             {(port.mac, ip_addr) for ip_addr in port.ipv6_addresses})
-        self._initialize_egress_ipv6_icmp(port, allowed_pairs)
-        for mac_addr, ip_addr in allowed_pairs:
+        self._initialize_egress_ipv6_icmp(port, allowed_mac_ipv6_pairs)
+        for mac_addr, ip_addr in allowed_mac_ipv6_pairs:
             self._add_flow(
                 table=ovs_consts.BASE_EGRESS_TABLE,
                 priority=65,
@@ -1071,21 +1111,30 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             )
 
         # DHCP discovery
-        for dl_type, src_port, dst_port in (
-                (lib_const.ETHERTYPE_IP, 68, 67),
-                (lib_const.ETHERTYPE_IPV6, 546, 547)):
-            self._add_flow(
-                table=ovs_consts.BASE_EGRESS_TABLE,
-                priority=80,
-                reg_port=port.ofport,
-                in_port=port.ofport,
-                dl_type=dl_type,
-                nw_proto=lib_const.PROTO_NUM_UDP,
-                tp_src=src_port,
-                tp_dst=dst_port,
-                actions='resubmit(,{:d})'.format(
-                    ovs_consts.ACCEPT_OR_INGRESS_TABLE)
-            )
+        additional_ipv4_filters = [
+            {"dl_src": mac, "nw_src": ip}
+            for mac, ip in (*allowed_mac_ipv4_pairs,
+                            (port.mac, '0.0.0.0'),)]
+        additional_ipv6_filters = [
+            {"dl_src": mac, "ipv6_src": ip}
+            for mac, ip in allowed_mac_ipv6_pairs]
+        for dl_type, src_port, dst_port, filters_list in (
+                (lib_const.ETHERTYPE_IP, 68, 67, additional_ipv4_filters),
+                (lib_const.ETHERTYPE_IPV6, 546, 547, additional_ipv6_filters)):
+            for additional_filters in filters_list:
+                self._add_flow(
+                    table=ovs_consts.BASE_EGRESS_TABLE,
+                    priority=80,
+                    reg_port=port.ofport,
+                    in_port=port.ofport,
+                    dl_type=dl_type,
+                    **additional_filters,
+                    nw_proto=lib_const.PROTO_NUM_UDP,
+                    tp_src=src_port,
+                    tp_dst=dst_port,
+                    actions='resubmit(,{:d})'.format(
+                        ovs_consts.ACCEPT_OR_INGRESS_TABLE)
+                )
         # Ban dhcp service running on an instance
         for dl_type, src_port, dst_port in (
                 (lib_const.ETHERTYPE_IP, 67, 68),
@@ -1518,19 +1567,17 @@ class OVSFirewallDriver(firewall.FirewallDriver):
     def delete_all_port_flows(self, port):
         """Delete all flows for given port"""
         for mac_addr in port.all_allowed_macs:
-            self._strict_delete_flow(priority=90,
-                                     table=ovs_consts.TRANSIENT_TABLE,
-                                     dl_dst=mac_addr,
-                                     dl_vlan=port.vlan_tag)
+            self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
+                               dl_dst=mac_addr,
+                               dl_vlan=port.vlan_tag)
             self.delete_physical_direct_flow(mac_addr, port.segment_id)
             self._delete_flows(table=ovs_consts.ACCEPT_OR_INGRESS_TABLE,
                                dl_dst=mac_addr, reg_net=port.vlan_tag)
 
         self.delete_accepted_egress_direct_flow(
             port.mac, port.vlan_tag)
-        self._strict_delete_flow(priority=100,
-                                 table=ovs_consts.TRANSIENT_TABLE,
-                                 in_port=port.ofport)
+        self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
+                           in_port=port.ofport)
         self._delete_flows(reg_port=port.ofport)
 
     def delete_flows_for_flow_state(
@@ -1565,4 +1612,4 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             # will not match with the ip flow's cookie so OVS won't actually
             # delete the flow
             flow['cookie'] = ovs_lib.COOKIE_ANY
-            self._delete_flows(deferred=False, **flow)
+            self._delete_flows(**flow)
