@@ -15,6 +15,7 @@
 import collections
 import functools
 import re
+import threading
 import uuid
 
 from neutron_lib import constants as n_const
@@ -48,6 +49,8 @@ OVN_VIF_PORT_TYPES = ("", "external", )
 MetadataPortInfo = collections.namedtuple('MetadataPortInfo', ['mac',
                                                                'ip_addresses'])
 
+OVN_METADATA_UUID_NAMESPACE = uuid.UUID('d34bf9f6-da32-4871-9af8-15a4626b41ab')
+
 
 def _sync_lock(f):
     """Decorator to block all operations for a global sync call."""
@@ -200,12 +203,31 @@ class MetadataAgent(object):
         self._process_monitor = external_process.ProcessMonitor(
             config=self.conf,
             resource_type='metadata')
+        self._sb_idl = None
+        self._post_fork_event = threading.Event()
+
+    @property
+    def sb_idl(self):
+        if not self._sb_idl:
+            self._post_fork_event.wait()
+        return self._sb_idl
+
+    @sb_idl.setter
+    def sb_idl(self, val):
+        self._sb_idl = val
 
     def _load_config(self):
         self.chassis = self._get_own_chassis_name()
+        try:
+            self.chassis_id = uuid.UUID(self.chassis)
+        except ValueError:
+            # OVS system-id could be a non UUID formatted string.
+            self.chassis_id = uuid.uuid5(OVN_METADATA_UUID_NAMESPACE,
+                                         self.chassis)
+
         self.ovn_bridge = self._get_ovn_bridge()
-        LOG.debug("Loaded chassis %s and ovn bridge %s.",
-                  self.chassis, self.ovn_bridge)
+        LOG.info("Loaded chassis name %s (UUID: %s) and ovn bridge %s.",
+                 self.chassis, self.chassis_id, self.ovn_bridge)
 
     @_sync_lock
     def resync(self):
@@ -221,11 +243,6 @@ class MetadataAgent(object):
         self.ovs_idl = ovsdb.MetadataAgentOvsIdl().start()
         self._load_config()
 
-        # Launch the server that will act as a proxy between the VM's and Nova.
-        proxy = metadata_server.UnixDomainMetadataProxy(self.conf,
-                self.chassis)
-        proxy.run()
-
         tables = ('Encap', 'Port_Binding', 'Datapath_Binding', 'SB_Global',
                   'Chassis')
         events = (PortBindingChassisCreatedEvent(self),
@@ -237,6 +254,7 @@ class MetadataAgent(object):
         # Chassis table.
         # Open the connection to OVN SB database.
         self.has_chassis_private = False
+        self._post_fork_event.clear()
         try:
             self.sb_idl = ovsdb.MetadataAgentOvnSbIdl(
                 chassis=self.chassis, tables=tables + ('Chassis_Private', ),
@@ -247,13 +265,21 @@ class MetadataAgent(object):
                 chassis=self.chassis, tables=tables,
                 events=events + (ChassisCreateEvent(self), )).start()
 
+        # Now IDL connections can be safely used.
+        self._post_fork_event.set()
+
+        # Launch the server that will act as a proxy between the VM's and Nova.
+        self._proxy = metadata_server.UnixDomainMetadataProxy(
+            self.conf, self.chassis, sb_idl=self.sb_idl)
+        self._proxy.run()
+
         # Do the initial sync.
         self.sync()
 
         # Register the agent with its corresponding Chassis
         self.register_metadata_agent()
 
-        proxy.wait()
+        self._proxy.wait()
 
     @tenacity.retry(
         wait=tenacity.wait_exponential(
@@ -263,9 +289,8 @@ class MetadataAgent(object):
         # NOTE(lucasagomes): db_add() will not overwrite the UUID if
         # it's already set.
         table = ('Chassis_Private' if self.has_chassis_private else 'Chassis')
-        chassis_id = uuid.UUID(self._get_own_chassis_name())
         # Generate unique, but consistent metadata id for chassis name
-        agent_id = uuid.uuid5(chassis_id, 'metadata_agent')
+        agent_id = uuid.uuid5(self.chassis_id, 'metadata_agent')
         ext_ids = {ovn_const.OVN_AGENT_METADATA_ID_KEY: str(agent_id)}
         self.sb_idl.db_add(table, self.chassis, 'external_ids',
                            ext_ids).execute(check_error=True)
@@ -335,8 +360,6 @@ class MetadataAgent(object):
         This function will shutdown metadata proxy if it's running and delete
         the VETH pair, the OVS port and the namespace.
         """
-        self.update_chassis_metadata_networks(datapath, remove=True)
-
         # TODO(dalvarez): Remove this in Y cycle when we are sure that all
         # namespaces will be created with the Neutron network UUID and not
         # anymore with the OVN datapath UUID.
@@ -515,7 +538,6 @@ class MetadataAgent(object):
             self.conf, bind_address=n_const.METADATA_V4_IP,
             network_id=net_name)
 
-        self.update_chassis_metadata_networks(net_name)
         return namespace
 
     def ensure_all_networks_provisioned(self):
@@ -541,30 +563,3 @@ class MetadataAgent(object):
                 namespaces.append(netns)
 
         return namespaces
-
-    # NOTE(lucasagomes): Even tho the metadata agent is a multi-process
-    # application, there's only one Southbound database IDL instance in
-    # the agent which handles the OVSDB events therefore we do not need
-    # the external=True parameter in the @synchronized decorator.
-    @lockutils.synchronized(CHASSIS_METADATA_LOCK)
-    def update_chassis_metadata_networks(self, datapath, remove=False):
-        """Update metadata networks hosted in this chassis.
-
-        Add or remove a datapath from the list of current datapaths that
-        we're currently serving metadata.
-        """
-        current_dps = self.sb_idl.get_chassis_metadata_networks(self.chassis)
-        updated = False
-        if remove:
-            if datapath in current_dps:
-                current_dps.remove(datapath)
-                updated = True
-        else:
-            if datapath not in current_dps:
-                current_dps.append(datapath)
-                updated = True
-
-        if updated:
-            with self.sb_idl.create_transaction(check_error=True) as txn:
-                txn.add(self.sb_idl.set_chassis_metadata_networks(
-                    self.chassis, current_dps))
