@@ -23,6 +23,7 @@ from neutron_lib.callbacks import resources
 from neutron_lib import constants
 from neutron_lib.db import api as db_api
 from neutron_lib import exceptions
+from neutron_lib.exceptions import agent as agent_exc
 from neutron_lib.plugins import directory
 from neutron_lib.plugins import utils as p_utils
 from oslo_config import cfg
@@ -32,7 +33,6 @@ import oslo_messaging
 from oslo_utils import excutils
 
 from neutron._i18n import _
-from neutron.common import constants as n_const
 from neutron.common import utils
 from neutron.db import provisioning_blocks
 from neutron.extensions import segment as segment_ext
@@ -71,22 +71,38 @@ class DhcpRpcCallback(object):
     #     1.6 - Removed get_active_networks. It's not used by reference
     #           DHCP agent since Havana, so similar rationale for not bumping
     #           the major version as above applies here too.
+    #     1.7 - Add get_networks
+    #     1.8 - Add get_dhcp_port
 
     target = oslo_messaging.Target(
-        namespace=n_const.RPC_NAMESPACE_DHCP_PLUGIN,
-        version='1.6')
+        namespace=constants.RPC_NAMESPACE_DHCP_PLUGIN,
+        version='1.8')
 
     def _get_active_networks(self, context, **kwargs):
         """Retrieve and return a list of the active networks."""
         host = kwargs.get('host')
         plugin = directory.get_plugin()
         if extensions.is_extension_supported(
-            plugin, constants.DHCP_AGENT_SCHEDULER_EXT_ALIAS):
+                plugin, constants.DHCP_AGENT_SCHEDULER_EXT_ALIAS):
             if cfg.CONF.network_auto_schedule:
                 plugin.auto_schedule_networks(context, host)
             nets = plugin.list_active_networks_on_active_dhcp_agent(
                 context, host)
         else:
+            # If no active DHCP agent or agent admin state is DOWN,
+            # return empty network list for RPC to avoid unexpected
+            # resource creation on remote host when the DHCP agent
+            # scheduler extension is not supported.
+            try:
+                agent = plugin._get_agent_by_type_and_host(
+                    context, constants.AGENT_TYPE_DHCP, host)
+            except agent_exc.AgentNotFoundByTypeHost:
+                LOG.debug("DHCP Agent not found on host %s", host)
+                return []
+            if not agent.admin_state_up:
+                LOG.debug("DHCP Agent admin state is down on host %s", host)
+                return []
+
             filters = dict(admin_state_up=[True])
             nets = plugin.get_networks(context, filters=filters)
         return nets
@@ -279,6 +295,7 @@ class DhcpRpcCallback(object):
                                                          hosts=[host])
         return len(agents) != 0
 
+    @oslo_messaging.expected_exceptions(exceptions.NetworkNotFound)
     @oslo_messaging.expected_exceptions(exceptions.IpAddressGenerationFailure)
     @db_api.retry_db_errors
     def update_dhcp_port(self, context, **kwargs):
@@ -294,10 +311,14 @@ class DhcpRpcCallback(object):
             if (old_port['device_id'] !=
                     constants.DEVICE_ID_RESERVED_DHCP_PORT and
                 old_port['device_id'] !=
-                    utils.get_dhcp_agent_device_id(network_id, host) or
-                not self._is_dhcp_agent_hosting_network(plugin, context, host,
-                                                        network_id)):
-                raise exceptions.DhcpPortInUse(port_id=port['id'])
+                    utils.get_dhcp_agent_device_id(network_id, host)):
+                return
+            if not self._is_dhcp_agent_hosting_network(plugin, context, host,
+                                                       network_id):
+                LOG.warning("The DHCP agent on %(host)s does not host the "
+                            "network %(net_id)s.", {"host": host,
+                                                    "net_id": network_id})
+                raise exceptions.NetworkNotFound(net_id=network_id)
             LOG.debug('Update dhcp port %(port)s '
                       'from %(host)s.',
                       {'port': port,
@@ -307,7 +328,13 @@ class DhcpRpcCallback(object):
             LOG.debug('Host %(host)s tried to update port '
                       '%(port_id)s which no longer exists.',
                       {'host': host, 'port_id': port['id']})
-            return None
+
+    @db_api.retry_db_errors
+    def get_dhcp_port(self, context, **kwargs):
+        """Retrieve the DHCP port"""
+        port_id = kwargs.get('port_id')
+        plugin = directory.get_plugin()
+        return plugin.get_port(context, port_id)
 
     @db_api.retry_db_errors
     def dhcp_ready_on_ports(self, context, port_ids):
@@ -315,3 +342,12 @@ class DhcpRpcCallback(object):
             provisioning_blocks.provisioning_complete(
                 context, port_id, resources.PORT,
                 provisioning_blocks.DHCP_ENTITY)
+
+    def get_networks(self, context, filters=None, fields=None):
+        """Retrieve and return a list of networks."""
+        # NOTE(adrianc): This RPC is being used by out of tree interface
+        # drivers, MultiInterfaceDriver and IPoIBInterfaceDriver, located in
+        # networking-mlnx.
+        plugin = directory.get_plugin()
+        return plugin.get_networks(
+            context, filters=filters, fields=fields)
