@@ -200,6 +200,7 @@ def main():
             'neutron.services.ovn_l3.plugin.OVNL3RouterPlugin',
             'neutron.services.segments.plugin.Plugin',
             'port_forwarding',
+            'qos'
         ]
     else:
         LOG.error('Invalid core plugin : ["%s"].', cfg.CONF.core_plugin)
@@ -225,8 +226,9 @@ def main():
     # and will fail with a KeyError if it isn't there
     core_plugin.mechanism_manager.mech_drivers['ovn'] = driver
     ovn_driver = driver.obj
-    ovn_driver._nb_ovn = ovn_api
-    ovn_driver._sb_ovn = ovn_sb_api
+    ovn_driver.nb_ovn = ovn_api
+    ovn_driver.sb_ovn = ovn_sb_api
+    ovn_driver._post_fork_event.set()
 
     synchronizer = ovn_db_sync.OvnNbSynchronizer(
         core_plugin, ovn_api, ovn_sb_api, mode, ovn_driver)
