@@ -614,26 +614,6 @@ def get_network_name_from_datapath(datapath):
     return datapath.external_ids['name'].replace('neutron-', '')
 
 
-def connection_config_to_target_string(connection_config):
-    """Converts the Neutron NB/SB connection parameter to the OVN target string
-
-    :param connection_config: Neutron OVN config parameter for the OVN NB or SB
-                              database. See "ovn_sb_connection" or
-                              "ovn_nb_connection" params.
-    :returns: (String) OVN NB/SB ``connection.target`` column value.
-    """
-    regex = re.compile(r'^(?P<proto>\w+)\:((?P<ip>.+)\:(?P<port>\d+)|'
-                       r'(?P<file>[\w\/\.]+))')
-    m = regex.match(connection_config)
-    if m:
-        _dict = m.groupdict()
-        if _dict['ip'] and _dict['port']:
-            return ('p' + _dict['proto'] + ':' + _dict['port'] + ':' +
-                    _dict['ip'])
-        elif _dict['file']:
-            return 'p' + _dict['proto'] + ':' + _dict['file']
-
-
 def is_port_external(port):
     # This port is represented in OVN DB as lsp.type=external
     capabilities = []
