@@ -100,6 +100,23 @@ OPTS = [
                       '(by default), the user executing the L3 agent will be '
                       'passed. If "root" specified, because radvd is spawned '
                       'as root, no "username" parameter will be passed.')),
+    cfg.BoolOpt('cleanup_on_shutdown', default=False,
+                help=_('Delete all routers on L3 agent shutdown. For L3 HA '
+                       'routers it includes a shutdown of keepalived and '
+                       'the state change monitor. NOTE: Setting to True '
+                       'could affect the data plane when stopping or '
+                       'restarting the L3 agent.')),
+    cfg.BoolOpt('keepalived_use_no_track',
+                default=True,
+                deprecated_for_removal=True,
+                deprecated_reason='By keepalived version detection introduced '
+                                  'by https://review.opendev.org/757620 there '
+                                  'is no need for this config option. To be '
+                                  'removed in X.',
+                help=_('If keepalived without support for "no_track" option '
+                       'is used, this should be set to False. '
+                       'Support for this option was introduced in keepalived '
+                       '2.x'))
 ]
 
 
