@@ -100,12 +100,12 @@ def _get_standard_attr_id(context, resource_uuid, resource_type):
 @db_api.retry_if_session_inactive()
 def create_initial_revision(context, resource_uuid, resource_type,
                             revision_number=INITIAL_REV_NUM,
-                            may_exist=False):
+                            may_exist=False, std_attr_id=None):
     LOG.debug('create_initial_revision uuid=%s, type=%s, rev=%s',
               resource_uuid, resource_type, revision_number)
     db_func = context.session.merge if may_exist else context.session.add
-    with context.session.begin(subtransactions=True):
-        std_attr_id = _get_standard_attr_id(
+    with db_api.CONTEXT_WRITER.using(context):
+        std_attr_id = std_attr_id or _get_standard_attr_id(
             context, resource_uuid, resource_type)
         row = ovn_models.OVNRevisionNumbers(
             resource_uuid=resource_uuid, resource_type=resource_type,
@@ -116,7 +116,7 @@ def create_initial_revision(context, resource_uuid, resource_type,
 @db_api.retry_if_session_inactive()
 def delete_revision(context, resource_uuid, resource_type):
     LOG.debug('delete_revision(%s)', resource_uuid)
-    with context.session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         row = context.session.query(ovn_models.OVNRevisionNumbers).filter_by(
             resource_uuid=resource_uuid,
             resource_type=resource_type).one_or_none()
@@ -124,7 +124,7 @@ def delete_revision(context, resource_uuid, resource_type):
             context.session.delete(row)
 
 
-def _ensure_revision_row_exist(context, resource, resource_type):
+def _ensure_revision_row_exist(context, resource, resource_type, std_attr_id):
     """Ensure the revision row exists.
 
     Ensure the revision row exist before we try to bump its revision
@@ -136,22 +136,24 @@ def _ensure_revision_row_exist(context, resource, resource_type):
     # deal with objects that already existed before the sync work. I believe
     # that we can remove this method after few development cycles. Or,
     # if we decide to make a migration script as well.
-    with context.session.begin(subtransactions=True):
-        if not context.session.query(ovn_models.OVNRevisionNumbers).filter_by(
-                resource_uuid=resource['id'],
-                resource_type=resource_type).one_or_none():
-            LOG.warning(
-                'No revision row found for %(res_uuid)s (type: '
-                '%(res_type)s) when bumping the revision number. '
-                'Creating one.', {'res_uuid': resource['id'],
-                                  'res_type': resource_type})
-            create_initial_revision(context, resource['id'], resource_type)
+    if context.session.query(ovn_models.OVNRevisionNumbers).filter_by(
+            resource_uuid=resource['id'],
+            resource_type=resource_type).one_or_none():
+        return
+
+    LOG.warning(
+        'No revision row found for %(res_uuid)s (type: '
+        '%(res_type)s) when bumping the revision number. '
+        'Creating one.', {'res_uuid': resource['id'],
+                          'res_type': resource_type})
+    create_initial_revision(context, resource['id'], resource_type,
+                            std_attr_id=std_attr_id)
 
 
 @db_api.retry_if_session_inactive()
 def get_revision_row(context, resource_uuid):
     try:
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_READER.using(context):
             return context.session.query(
                 ovn_models.OVNRevisionNumbers).filter_by(
                 resource_uuid=resource_uuid).one()
@@ -162,10 +164,17 @@ def get_revision_row(context, resource_uuid):
 @db_api.retry_if_session_inactive()
 def bump_revision(context, resource, resource_type):
     revision_number = ovn_utils.get_revision_number(resource, resource_type)
-    with context.session.begin(subtransactions=True):
-        _ensure_revision_row_exist(context, resource, resource_type)
-        std_attr_id = _get_standard_attr_id(
-            context, resource['id'], resource_type)
+    with db_api.CONTEXT_WRITER.using(context):
+        # NOTE(ralonsoh): "resource" could be a dict or an OVO.
+        try:
+            std_attr_id = resource.db_obj.standard_attr.id
+        except AttributeError:
+            std_attr_id = resource.get('standard_attr_id', None)
+        _ensure_revision_row_exist(context, resource, resource_type,
+                                   std_attr_id)
+        std_attr_id = (std_attr_id or
+                       _get_standard_attr_id(context, resource['id'],
+                                             resource_type))
         row = context.session.merge(ovn_models.OVNRevisionNumbers(
             standard_attr_id=std_attr_id, resource_uuid=resource['id'],
             resource_type=resource_type))
@@ -195,7 +204,7 @@ def get_inconsistent_resources(context):
                          whens=MAINTENANCE_CREATE_UPDATE_TYPE_ORDER)
     time_ = (timeutils.utcnow() -
              datetime.timedelta(seconds=INCONSISTENCIES_OLDER_THAN))
-    with context.session.begin(subtransactions=True):
+    with db_api.CONTEXT_READER.using(context):
         query = context.session.query(ovn_models.OVNRevisionNumbers).join(
             standard_attr.StandardAttribute,
             ovn_models.OVNRevisionNumbers.standard_attr_id ==
@@ -224,6 +233,6 @@ def get_deleted_resources(context):
     """
     sort_order = sa.case(value=ovn_models.OVNRevisionNumbers.resource_type,
                          whens=MAINTENANCE_DELETE_TYPE_ORDER)
-    with context.session.begin(subtransactions=True):
+    with db_api.CONTEXT_READER.using(context):
         return context.session.query(ovn_models.OVNRevisionNumbers).filter_by(
             standard_attr_id=None).order_by(sort_order).all()
