@@ -13,6 +13,7 @@
 #
 
 import abc
+import copy
 
 from oslo_config import cfg
 from oslo_utils import timeutils
@@ -23,6 +24,12 @@ from neutron.common.ovn import utils as ovn_utils
 from neutron.common import utils
 
 
+class DeletedChassis(object):
+    external_ids = {}
+    hostname = '("Chassis" register deleted)'
+    name = '("Chassis" register deleted)'
+
+
 class NeutronAgent(abc.ABC):
     types = {}
 
@@ -45,9 +52,12 @@ class NeutronAgent(abc.ABC):
     def chassis_from_private(chassis_private):
         try:
             return chassis_private.chassis[0]
-        except (AttributeError, IndexError):
+        except AttributeError:
             # No Chassis_Private support, just use Chassis
             return chassis_private
+        except IndexError:
+            # Chassis register has been deleted but not Chassis_Private.
+            return DeletedChassis
 
     @property
     def chassis(self):
@@ -77,8 +87,8 @@ class NeutronAgent(abc.ABC):
         if self.set_down:
             return False
         # TODO(twilson) Determine if we can go back to just checking:
-        # if self.driver._nb_ovn.nb_global.nb_cfg == self.nb_cfg:
-        if self.driver._nb_ovn.nb_global.nb_cfg - self.nb_cfg <= 1:
+        # if self.driver.nb_ovn.nb_global.nb_cfg == self.nb_cfg:
+        if self.driver.nb_ovn.nb_global.nb_cfg - self.nb_cfg <= 1:
             return True
         now = timeutils.utcnow(with_timezone=True)
         if (now - self.updated_at).total_seconds() < cfg.CONF.agent_down_time:
@@ -205,7 +215,10 @@ class AgentCache:
         self.driver = driver
 
     def __iter__(self):
-        return iter(self.agents.values())
+        # Copying self.agents will avoid any issue during the iteration if an
+        # agent is added or deleted.
+        _agents = copy.copy(self.agents)
+        return iter(_agents.values())
 
     def __getitem__(self, key):
         return self.agents[key]
@@ -229,4 +242,4 @@ class AgentCache:
         agent_ids = {cls.id_from_chassis_private(chassis_private)
                      for cls in NeutronAgent.types.values()}
         # Return the cached agents of agent_ids whose keys are in the cache
-        return (self.agents[id_] for id_ in agent_ids & self.agents.keys())
+        return (agent for agent in self if agent.agent_id in agent_ids)
