@@ -16,6 +16,7 @@ import atexit
 import copy
 import datetime
 import functools
+import multiprocessing
 import operator
 import signal
 import threading
@@ -34,12 +35,12 @@ from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import directory
 from neutron_lib.plugins.ml2 import api
 from neutron_lib.utils import helpers
+from oslo_concurrency import lockutils
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_db import exception as os_db_exc
 from oslo_log import log
 from oslo_utils import timeutils
-from ovsdbapp.backend.ovs_idl import idlutils
 
 from neutron._i18n import _
 from neutron.common.ovn import acl as ovn_acl
@@ -47,7 +48,6 @@ from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import exceptions as ovn_exceptions
 from neutron.common.ovn import extensions as ovn_extensions
 from neutron.common.ovn import utils as ovn_utils
-from neutron.common import utils as n_utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db
 from neutron.db import ovn_revision_numbers_db
@@ -70,11 +70,6 @@ import neutron.wsgi
 
 
 LOG = log.getLogger(__name__)
-METADATA_READY_WAIT_TIMEOUT = 15
-
-
-class MetadataServiceReadyWaitTimeoutException(Exception):
-    pass
 
 
 class OVNPortUpdateError(n_exc.BadRequest):
@@ -117,13 +112,11 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._plugin_property = None
         self._ovn_client_inst = None
         self._maintenance_thread = None
+        self._hash_ring_thread = None
+        self._hash_ring_probe_event = multiprocessing.Event()
         self.node_uuid = None
         self.hash_ring_group = ovn_const.HASH_RING_ML2_GROUP
         self.sg_enabled = ovn_acl.is_sg_enabled()
-        # NOTE(lucasagomes): _clean_hash_ring() must be called before
-        # self.subscribe() to avoid processes racing when adding or
-        # deleting nodes from the Hash Ring during service initialization
-        self._clean_hash_ring()
         self._post_fork_event = threading.Event()
         if cfg.CONF.SECURITYGROUP.firewall_driver:
             LOG.warning('Firewall driver configuration is ignored')
@@ -176,6 +169,16 @@ class OVNMechanismDriver(api.MechanismDriver):
     def sb_ovn(self, val):
         self._sb_ovn = val
 
+    def get_supported_vif_types(self):
+        vif_types = set()
+        for ch in self.sb_ovn.chassis_list().execute(check_error=True):
+            dp_type = ch.external_ids.get('datapath-type', '')
+            if dp_type == ovn_const.CHASSIS_DATAPATH_NETDEV:
+                vif_types.add(portbindings.VIF_TYPE_VHOST_USER)
+            else:
+                vif_types.add(portbindings.VIF_TYPE_OVS)
+        return list(vif_types)
+
     def check_vlan_transparency(self, context):
         """OVN driver vlan transparency support."""
         vlan_transparency_network_types = [
@@ -221,6 +224,10 @@ class OVNMechanismDriver(api.MechanismDriver):
     def supported_extensions(self, extensions):
         return set(ovn_extensions.ML2_SUPPORTED_API_EXTENSIONS) & extensions
 
+    @staticmethod
+    def provider_network_attribute_updates_supported():
+        return [provider_net.SEGMENTATION_ID]
+
     def subscribe(self):
         registry.subscribe(self.pre_fork_initialize,
                            resources.PROCESS,
@@ -249,6 +256,9 @@ class OVNMechanismDriver(api.MechanismDriver):
             registry.subscribe(self._create_security_group,
                                resources.SECURITY_GROUP,
                                events.AFTER_CREATE)
+            registry.subscribe(self._delete_security_group_precommit,
+                               resources.SECURITY_GROUP,
+                               events.PRECOMMIT_DELETE)
             registry.subscribe(self._delete_security_group,
                                resources.SECURITY_GROUP,
                                events.AFTER_DELETE)
@@ -272,7 +282,6 @@ class OVNMechanismDriver(api.MechanismDriver):
         atexit.register(self._clean_hash_ring)
         signal.signal(signal.SIGTERM, self._clean_hash_ring)
         self._create_neutron_pg_drop()
-        self._set_inactivity_probe()
 
     def _create_neutron_pg_drop(self):
         """Create neutron_pg_drop Port Group.
@@ -309,34 +318,37 @@ class OVNMechanismDriver(api.MechanismDriver):
                 else:
                     raise
 
-    def _set_inactivity_probe(self):
-        """Set 'connection.inactivity_probe' in NB and SB databases"""
-        inactivity_probe = ovn_conf.get_ovn_ovsdb_probe_interval()
-        dbs = [(ovn_conf.get_ovn_nb_connection(), self.nb_schema_helper,
-                impl_idl_ovn.OvsdbNbOvnIdl),
-               (ovn_conf.get_ovn_sb_connection(), self.sb_schema_helper,
-                impl_idl_ovn.OvsdbSbOvnIdl)]
-        for connection, schema, klass in dbs:
-            target = ovn_utils.connection_config_to_target_string(connection)
-            if not target:
-                continue
-
-            idl = ovsdb_monitor.BaseOvnIdl.from_server(connection, schema)
-            with ovsdb_monitor.short_living_ovsdb_api(klass, idl) as idl_api:
-                conn = idlutils.row_by_value(idl_api, 'Connection', 'target',
-                                             target, None)
-                if conn:
-                    idl_api.db_set(
-                        'Connection', target,
-                        ('inactivity_probe', int(inactivity_probe))).execute(
-                        check_error=True)
-
     @staticmethod
     def should_post_fork_initialize(worker_class):
         return worker_class in (neutron.wsgi.WorkerService,
                                 worker.MaintenanceWorker,
                                 service.RpcWorker)
 
+    @lockutils.synchronized('hash_ring_probe_lock', external=True)
+    def _setup_hash_ring(self):
+        """Setup the hash ring.
+
+        The first worker to acquire the lock is responsible for cleaning
+        the hash ring from previous runs as well as start the probing
+        thread for this host. Subsequently workers just need to register
+        themselves to the hash ring.
+        """
+        admin_context = n_context.get_admin_context()
+        if not self._hash_ring_probe_event.is_set():
+            self._clean_hash_ring()
+            self.node_uuid = ovn_hash_ring_db.add_node(admin_context,
+                                                       self.hash_ring_group)
+            self._hash_ring_thread = maintenance.MaintenanceThread()
+            self._hash_ring_thread.add_periodics(
+                maintenance.HashRingHealthCheckPeriodics(
+                    self.hash_ring_group))
+            self._hash_ring_thread.start()
+            LOG.info("Hash Ring probing thread has started")
+            self._hash_ring_probe_event.set()
+        else:
+            self.node_uuid = ovn_hash_ring_db.add_node(admin_context,
+                                                       self.hash_ring_group)
+
     def post_fork_initialize(self, resource, event, trigger, payload=None):
         # Initialize API/Maintenance workers with OVN IDL connections
         worker_class = ovn_utils.get_method_class(trigger)
@@ -348,9 +360,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._ovn_client_inst = None
 
         if worker_class == neutron.wsgi.WorkerService:
-            admin_context = n_context.get_admin_context()
-            self.node_uuid = ovn_hash_ring_db.add_node(admin_context,
-                                                       self.hash_ring_group)
+            self._setup_hash_ring()
 
         n_agent.AgentCache(self)  # Initialize singleton agent cache
         self.nb_ovn, self.sb_ovn = impl_idl_ovn.get_ovn_idls(self, trigger)
@@ -391,10 +401,8 @@ class OVNMechanismDriver(api.MechanismDriver):
             self._maintenance_thread = maintenance.MaintenanceThread()
             self._maintenance_thread.add_periodics(
                 maintenance.DBInconsistenciesPeriodics(self._ovn_client))
-            self._maintenance_thread.add_periodics(
-                maintenance.HashRingHealthCheckPeriodics(
-                    self.hash_ring_group))
             self._maintenance_thread.start()
+            LOG.info("Maintenance task thread has started")
 
     def _wait_for_pg_drop_event(self):
         """Wait for event that occurs when neutron_pg_drop Port Group exists.
@@ -430,6 +438,13 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._ovn_client.delete_security_group(kwargs['context'],
                                                security_group_id)
 
+    def _delete_security_group_precommit(self, resource, event, trigger,
+                                         security_group_id, **kwargs):
+        context = n_context.get_admin_context()
+        for sg_rule in self._plugin.get_security_group_rules(
+                context, filters={'remote_group_id': [security_group_id]}):
+            self._ovn_client.delete_security_group_rule(context, sg_rule)
+
     def _update_security_group(self, resource, event, trigger,
                                security_group, **kwargs):
         # OVN doesn't care about updates to security groups, only if they
@@ -1084,7 +1099,6 @@ class OVNMechanismDriver(api.MechanismDriver):
         LOG.info("OVN reports status up for port: %s", port_id)
 
         self._update_dnat_entry_if_needed(port_id)
-        self._wait_for_metadata_provisioned_if_needed(port_id)
 
         admin_context = n_context.get_admin_context()
         provisioning_blocks.provisioning_complete(
@@ -1186,61 +1200,6 @@ class OVNMechanismDriver(api.MechanismDriver):
                  if phynet in phynets}
         segment_service_db.map_segment_to_hosts(context, segment.id, hosts)
 
-    def _wait_for_metadata_provisioned_if_needed(self, port_id):
-        """Wait for metadata service to be provisioned.
-
-        Wait until metadata service has been setup for this port in the chassis
-        it resides. If metadata is disabled or DHCP is not enabled for its
-        subnets, this function will return right away.
-        """
-        if ovn_conf.is_ovn_metadata_enabled() and self.sb_ovn:
-            # Wait until metadata service has been setup for this port in the
-            # chassis it resides.
-            result = (
-                self.sb_ovn.get_logical_port_chassis_and_datapath(port_id))
-            if not result:
-                LOG.warning("Logical port %s doesn't exist in OVN", port_id)
-                return
-            chassis, datapath = result
-            if not chassis:
-                LOG.warning("Logical port %s is not bound to a "
-                            "chassis", port_id)
-                return
-
-            # Check if the port belongs to some IPv4 subnet with DHCP enabled.
-            context = n_context.get_admin_context()
-            port = self._plugin.get_port(context, port_id)
-            port_subnet_ids = set(
-                ip['subnet_id'] for ip in port['fixed_ips'] if
-                n_utils.get_ip_version(ip['ip_address']) == const.IP_VERSION_4)
-            if not port_subnet_ids:
-                # The port doesn't belong to any IPv4 subnet
-                return
-
-            subnets = self._plugin.get_subnets(context, filters=dict(
-                network_id=[port['network_id']], ip_version=[4],
-                enable_dhcp=True))
-
-            subnet_ids = set(
-                s['id'] for s in subnets if s['id'] in port_subnet_ids)
-            if not subnet_ids:
-                return
-
-            try:
-                n_utils.wait_until_true(
-                    lambda: datapath in
-                    self.sb_ovn.get_chassis_metadata_networks(chassis),
-                    timeout=METADATA_READY_WAIT_TIMEOUT,
-                    exception=MetadataServiceReadyWaitTimeoutException)
-            except MetadataServiceReadyWaitTimeoutException:
-                # If we reach this point it means that metadata agent didn't
-                # provision the datapath for this port on its chassis. Either
-                # the agent is not running or it crashed. We'll complete the
-                # provisioning block though.
-                LOG.warning("Metadata service is not ready for port %s, check"
-                            " neutron-ovn-metadata-agent status/logs.",
-                            port_id)
-
     def check_segment_for_agent(self, segment, agent):
         """Check if the OVN controller agent br mappings has segment physnet
 
