@@ -427,7 +427,7 @@ class LogicalSwitchPortUpdateUpEvent(row_event.RowEvent):
         events = (self.ROW_UPDATE,)
         super(LogicalSwitchPortUpdateUpEvent, self).__init__(
             events, table, (('up', '=', True),),
-            old_conditions=(('up', '=', False),))
+            old_conditions=(('up', '!=', True),))
         self.event_name = 'LogicalSwitchPortUpdateUpEvent'
 
     def run(self, event, row, old):
@@ -776,7 +776,6 @@ class OvnSbIdl(OvnIdlDistributedLock):
         helper.register_table('Encap')
         helper.register_table('Port_Binding')
         helper.register_table('Datapath_Binding')
-        helper.register_table('Connection')
         helper.register_columns('SB_Global', ['external_ids'])
         try:
             return cls(driver, connection_string, helper, leader_only=False)
