@@ -20,6 +20,7 @@ from neutron.agent.linux import iptables_firewall
 from neutron.agent.linux import iptables_manager
 from neutron.cmd.sanity import checks
 from neutron.common import utils as common_utils
+from neutron.tests import base as tests_base
 from neutron.tests.common import net_helpers
 from neutron.tests.fullstack import base
 from neutron.tests.fullstack.resources import environment
@@ -108,6 +109,7 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
     # NOTE(toshii): As a firewall_driver can interfere with others,
     # the recommended way to add test is to expand this method, not
     # adding another.
+    @tests_base.unstable_test("bug 1742401")
     def test_securitygroup(self):
         """Tests if a security group rules are working, by confirming
         that 0. traffic is allowed when port security is disabled,
@@ -133,15 +135,15 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
             tenant_uuid, subnet_cidr)
 
         # 0. check that traffic is allowed when port security is disabled
+        vms[0].block_until_ping(vms[1].ip)
+        vms[0].block_until_ping(vms[2].ip)
+        vms[1].block_until_ping(vms[2].ip)
         self.assert_connection(
             vms[1].namespace, vms[0].namespace, vms[0].ip, 3333,
             net_helpers.NetcatTester.TCP)
         self.assert_connection(
             vms[2].namespace, vms[0].namespace, vms[0].ip, 3333,
             net_helpers.NetcatTester.TCP)
-        vms[0].block_until_ping(vms[1].ip)
-        vms[0].block_until_ping(vms[2].ip)
-        vms[1].block_until_ping(vms[2].ip)
 
         # Apply security groups to the ports
         for port, sg in zip(ports, self.index_to_sg):
@@ -689,3 +691,18 @@ class SecurityGroupRulesTest(base.BaseFullStackTestCase):
 
         self.assertRaises(nc_exc.OverQuotaClient,
                           self.safe_client.create_security_group, project_id)
+
+    def test_normalized_cidr_in_rule(self):
+        project_id = uuidutils.generate_uuid()
+        sg = self.safe_client.create_security_group(project_id)
+
+        rule = self.safe_client.create_security_group_rule(
+            project_id, sg['id'], direction='ingress',
+            remote_ip_prefix='10.0.0.34/24')
+        self.assertEqual('10.0.0.0/24', rule['normalized_cidr'])
+        self.assertEqual('10.0.0.34/24', rule['remote_ip_prefix'])
+
+        rule = self.safe_client.create_security_group_rule(
+            project_id, sg['id'], direction='ingress')
+        self.assertIsNone(rule['normalized_cidr'])
+        self.assertIsNone(rule['remote_ip_prefix'])
