@@ -12,26 +12,33 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import functools
 from unittest import mock
 
 import fixtures as og_fixtures
+from neutron_lib.api.definitions import allowedaddresspairs
+from neutron_lib.api.definitions import external_net
+from neutron_lib.api.definitions import portbindings
+from neutron_lib.plugins import constants as plugin_constants
+from neutron_lib.plugins import directory
 from oslo_concurrency import processutils
 from oslo_utils import uuidutils
+from ovsdbapp.backend.ovs_idl import event
+from ovsdbapp.backend.ovs_idl import idlutils
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common import utils as n_utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db as db_hash_ring
 from neutron.plugins.ml2.drivers.ovn.agent import neutron_agent
+from neutron.plugins.ml2.drivers.ovn.mech_driver import mech_driver
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
 from neutron.tests.functional import base
+from neutron.tests.functional.resources.ovsdb import events as test_events
 from neutron.tests.functional.resources.ovsdb import fixtures
 from neutron.tests.functional.resources import process
-from neutron_lib.api.definitions import portbindings
-from neutron_lib.plugins import constants as plugin_constants
-from neutron_lib.plugins import directory
-from ovsdbapp.backend.ovs_idl import event
-from ovsdbapp.backend.ovs_idl import idlutils
+from neutron.tests.unit.api import test_extensions
+from neutron.tests.unit.extensions import test_l3
 
 
 class WaitForDataPathBindingCreateEvent(event.WaitEvent):
@@ -80,16 +87,21 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         super(TestNBDbMonitor, self).setUp()
         self.chassis = self.add_fake_chassis('ovs-host1')
         self.l3_plugin = directory.get_plugin(plugin_constants.L3)
+        self.net = self._make_network(self.fmt, 'net1', True)
+        self._make_subnet(self.fmt, self.net, '20.0.0.1', '20.0.0.0/24',
+                          ip_version=4)
 
-    def create_port(self):
-        net = self._make_network(self.fmt, 'net1', True)
-        self._make_subnet(self.fmt, net, '20.0.0.1',
-                          '20.0.0.0/24', ip_version=4)
-        arg_list = ('device_owner', 'device_id', portbindings.HOST_ID)
-        host_arg = {'device_owner': 'compute:nova',
+    def create_port(self, device_owner='compute:nova', host='ovs-host1',
+                    allowed_address_pairs=None):
+        allowed_address_pairs = allowed_address_pairs or []
+        arg_list = ('device_owner', 'device_id', portbindings.HOST_ID,
+                    allowedaddresspairs.ADDRESS_PAIRS)
+        host_arg = {'device_owner': device_owner,
                     'device_id': uuidutils.generate_uuid(),
-                    portbindings.HOST_ID: 'ovs-host1'}
-        port_res = self._create_port(self.fmt, net['network']['id'],
+                    portbindings.HOST_ID: host,
+                    allowedaddresspairs.ADDRESS_PAIRS: allowed_address_pairs
+                    }
+        port_res = self._create_port(self.fmt, self.net['network']['id'],
                                      arg_list=arg_list, **host_arg)
         port = self.deserialize(self.fmt, port_res)['port']
         return port
@@ -191,18 +203,12 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
             lambda: not self._check_mac_binding_exists(macb_id),
             timeout=15, sleep=1)
 
-    def _get_port_uuid(self, port_id):
-        sb_port = self.sb_api.db_find(
-            'Port_Binding', ('logical_port', '=', port_id)).execute()[0]
-        return sb_port['_uuid']
-
     def _test_port_binding_and_status(self, port_id, action, status):
         # This function binds or unbinds port to chassis and
         # checks if port status matches with input status
         core_plugin = directory.get_plugin()
         self.sb_api.check_for_row_by_value_and_retry(
             'Port_Binding', 'logical_port', port_id)
-        port_uuid = self._get_port_uuid(port_id)
 
         def check_port_status(status):
             port = core_plugin.get_ports(
@@ -211,23 +217,8 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         if action == 'bind':
             self.sb_api.lsp_bind(port_id, self.chassis,
                                  may_exist=True).execute(check_error=True)
-            try:
-                self.sb_api.db_set('Port_Binding', port_uuid,
-                                   ('up', True)).execute(check_error=True)
-            except KeyError:
-                self.sb_api.db_set('Port_Binding', port_uuid,
-                                   ('chassis', 'host-1')).execute(
-                                       check_error=True)
-
         else:
             self.sb_api.lsp_unbind(port_id).execute(check_error=True)
-            try:
-                self.sb_api.db_set('Port_Binding', port_uuid,
-                                   ('up', False)).execute(check_error=True)
-            except KeyError:
-                self.sb_api.db_set('Port_Binding', port_uuid,
-                                   ('chassis', None)).execute(check_error=True)
-
         n_utils.wait_until_true(lambda: check_port_status(status))
 
     def test_port_up_down_events(self):
@@ -313,6 +304,55 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
                 "Fanout event didn't get handled expected %d times" %
                 (worker_num + 1)))
 
+    def _find_port_binding(self, port_id):
+        cmd = self.sb_api.db_find_rows('Port_Binding',
+                                       ('logical_port', '=', port_id))
+        rows = cmd.execute(check_error=True)
+        return rows[0] if rows else None
+
+    def _check_port_binding_type(self, port_id, port_type):
+        def is_port_binding_type(port_id, port_type):
+            bp = self._find_port_binding(port_id)
+            return port_type == bp.type
+
+        check = functools.partial(is_port_binding_type, port_id, port_type)
+        n_utils.wait_until_true(check, timeout=10)
+
+    def _check_port_virtual_parents(self, port_id, vparents):
+        def is_port_virtual_parents(port_id, vparents):
+            bp = self._find_port_binding(port_id)
+            return (vparents ==
+                    bp.options.get(ovn_const.LSP_OPTIONS_VIRTUAL_PARENTS_KEY))
+
+        check = functools.partial(is_port_virtual_parents, port_id, vparents)
+        n_utils.wait_until_true(check, timeout=10)
+
+    @mock.patch.object(mech_driver.OVNMechanismDriver,
+                       'update_virtual_port_host')
+    def test_virtual_port_host_update(self, mock_update_vip_host):
+        # NOTE: because we can't simulate traffic from a port, this check is
+        # not done in this test. This test checks the VIP host is unset when
+        # the port allowed address pairs are removed.
+        vip = self.create_port(device_owner='', host='')
+        vip_address = vip['fixed_ips'][0]['ip_address']
+        allowed_address_pairs = [{'ip_address': vip_address}]
+        port = self.create_port()
+        self._check_port_binding_type(vip['id'], '')
+
+        data = {'port': {'allowed_address_pairs': allowed_address_pairs}}
+        req = self.new_update_request('ports', data, port['id'])
+        req.get_response(self.api)
+        # This test checks that the VIP "Port_Binding" register gets the type
+        # and the corresponding "virtual-parents".
+        self._check_port_binding_type(vip['id'], ovn_const.LSP_TYPE_VIRTUAL)
+        mock_update_vip_host.assert_not_called()
+
+        data = {'port': {'allowed_address_pairs': []}}
+        req = self.new_update_request('ports', data, port['id'])
+        req.get_response(self.api)
+        self._check_port_binding_type(vip['id'], '')
+        mock_update_vip_host.assert_called_once_with(vip['id'], None)
+
 
 class TestNBDbMonitorOverTcp(TestNBDbMonitor):
     def get_ovsdb_server_protocol(self):
@@ -324,6 +364,71 @@ class TestNBDbMonitorOverSsl(TestNBDbMonitor):
         return 'ssl'
 
 
+class TestSBDbMonitor(base.TestOVNFunctionalBase, test_l3.L3NatTestCaseMixin):
+
+    def setUp(self, **kwargs):
+        super().setUp(**kwargs)
+        self.chassis = self.add_fake_chassis('ovs-host1',
+                                             enable_chassis_as_gw=True)
+        self.l3_plugin = directory.get_plugin(plugin_constants.L3)
+        ext_mgr = test_l3.L3TestExtensionManager()
+        self.ext_api = test_extensions.setup_extensions_middleware(ext_mgr)
+        self.handler = event.RowEventHandler()
+        self.sb_api.idl.notify = self.handler.notify
+
+    def _find_port_binding(self, port_type):
+        rows = self.sb_api.db_find_rows(
+            'Port_Binding', ('type', '=', port_type)).execute(check_error=True)
+        return rows[0] if rows else None
+
+    def test_router_port_binding(self):
+        # This test will check the router GW port creation and binding.
+        def check_ext_ids():
+            pb = self._find_port_binding(ovn_const.OVN_CHASSIS_REDIRECT)
+            _, lrp = list(
+                self.nb_api.tables['Logical_Router_Port'].rows.data.items())[0]
+            if pb.external_ids == {}:
+                # The current version of OVN installed in FT CI could not have
+                # [1]. In this case, the Port_Binding.external_ids value is an
+                # empty dictionary.
+                # [1]https://www.mail-archive.com/ovs-dev@openvswitch.org/
+                #    msg62836.html
+                return True
+            return lrp.external_ids == pb.external_ids
+
+        kwargs = {'arg_list': (external_net.EXTERNAL,),
+                  external_net.EXTERNAL: True}
+        ext_net = self._make_network(self.fmt, 'ext_net', True, **kwargs)
+        self._make_subnet(self.fmt, ext_net, '10.251.0.1', '10.251.0.0/24',
+                          enable_dhcp=True)
+        router = self._make_router(self.fmt, self._tenant_id)
+        row_event = test_events.WaitForCreatePortBindingEventPerType()
+        self.handler.watch_event(row_event)
+        self._add_external_gateway_to_router(router['router']['id'],
+                                             ext_net['network']['id'])
+        self.assertTrue(row_event.wait())
+
+        # Check SB pb.external_ids == NB lrp.external_ids
+        port_binding = self._find_port_binding(ovn_const.OVN_CHASSIS_REDIRECT)
+        try:
+            self.sb_api.db_set('Port_Binding', port_binding.uuid,
+                               ('up', True)).execute(check_error=True)
+        except KeyError:
+            # Current OVN version is prior to [1].
+            # [1]https://www.mail-archive.com/ovs-dev@openvswitch.org/
+            #    msg62836.html
+            return
+
+        try:
+            n_utils.wait_until_true(check_ext_ids, timeout=5)
+        except n_utils.WaitTimeout:
+            pb = self._find_port_binding(ovn_const.OVN_CHASSIS_REDIRECT)
+            _, lrp = list(
+                self.nb_api.tables['Logical_Router_Port'].rows.data.items())[0]
+            self.fail('pb.ext_ids: %s  --  lrp.ext_ids: %s' %
+                      (pb.external_ids, lrp.external_ids))
+
+
 class TestAgentMonitor(base.TestOVNFunctionalBase):
     FAKE_CHASSIS_HOST = 'fake-chassis-host'
 
