@@ -13,8 +13,8 @@
 #    under the License.
 
 import collections
+from unittest import mock
 
-import mock
 from neutron_lib import exceptions
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
@@ -521,9 +521,9 @@ class OVS_Lib_Test(base.BaseTestCase):
         self.br.ovsdb.list_ports.return_value.execute.return_value = [
             'qvo1', 'qvo2', 'qvo4']
         by_id = self.br.get_vifs_by_ids(['pid1', 'pid2', 'pid3', 'pid4'])
-        # pid3 isn't on bridge and pid4 doesn't have a valid ofport
+        # pid3 isn't on bridge
         self.assertIsNone(by_id['pid3'])
-        self.assertIsNone(by_id['pid4'])
+        self.assertEqual(-1, by_id['pid4'].ofport)
         self.assertEqual('pid1', by_id['pid1'].vif_id)
         self.assertEqual('qvo1', by_id['pid1'].port_name)
         self.assertEqual(1, by_id['pid1'].ofport)
@@ -535,6 +535,8 @@ class OVS_Lib_Test(base.BaseTestCase):
                        if_exists=True)])
 
     def test_get_port_ofport_retry(self):
+        # Increase this value to avoid a timeout during the test execution
+        self.br.ovsdb.ovsdb_connection.timeout = 10
         with mock.patch.object(
                 self.br, 'db_get_val',
                 side_effect=[[], [], [], [], 1]):
@@ -542,7 +544,7 @@ class OVS_Lib_Test(base.BaseTestCase):
 
     def test_get_port_ofport_retry_fails(self):
         # reduce timeout for faster execution
-        self.br.ovsdb_timeout = 1
+        self.br.ovsdb.ovsdb_connection.timeout = 1
         # after 7 calls the retry will timeout and raise
         with mock.patch.object(
                 self.br, 'db_get_val',
