@@ -12,7 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 import netaddr
 from neutron_lib import context
 from oslo_utils import uuidutils
@@ -21,6 +22,7 @@ from neutron.agent import resource_cache
 from neutron.api.rpc.callbacks import resources
 from neutron.api.rpc.handlers import securitygroups_rpc
 from neutron import objects
+from neutron.objects import address_group
 from neutron.objects.port.extensions import port_security as psec
 from neutron.objects import ports
 from neutron.objects import securitygroup
@@ -69,7 +71,7 @@ class SecurityGroupServerAPIShimTestCase(base.BaseTestCase):
         super(SecurityGroupServerAPIShimTestCase, self).setUp()
         objects.register_objects()
         resource_types = [resources.PORT, resources.SECURITYGROUP,
-                          resources.SECURITYGROUPRULE]
+                          resources.SECURITYGROUPRULE, resources.ADDRESSGROUP]
         self.rcache = resource_cache.RemoteResourceCache(resource_types)
         # prevent any server lookup attempts
         mock.patch.object(self.rcache, '_flood_cache_for_query').start()
@@ -92,6 +94,28 @@ class SecurityGroupServerAPIShimTestCase(base.BaseTestCase):
         self.rcache.record_resource_update(self.ctx, 'Port', p)
         return p
 
+    @mock.patch.object(address_group.AddressGroup, 'is_shared_with_tenant',
+                       return_value=False)
+    def _make_address_group_ovo(self, *args, **kwargs):
+        id = uuidutils.generate_uuid()
+        address_associations = [
+            address_group.AddressAssociation(
+                self.ctx,
+                address=netaddr.IPNetwork('10.0.0.1/32'),
+                address_group_id=id),
+            address_group.AddressAssociation(
+                self.ctx,
+                address=netaddr.IPNetwork('2001:db8::/32'),
+                address_group_id=id)
+        ]
+        ag = address_group.AddressGroup(self.ctx, id=id,
+                                        name='an-address-group',
+                                        description='An address group',
+                                        addresses=address_associations)
+        self.rcache.record_resource_update(self.ctx, resources.ADDRESSGROUP,
+                                           ag)
+        return ag
+
     @mock.patch.object(securitygroup.SecurityGroup, 'is_shared_with_tenant',
                        return_value=False)
     def _make_security_group_ovo(self, *args, **kwargs):
@@ -104,6 +128,8 @@ class SecurityGroupServerAPIShimTestCase(base.BaseTestCase):
             port_range_min=400,
             remote_group_id=attrs['id'],
             revision_number=1,
+            remote_address_group_id=kwargs.get('remote_address_group_id',
+                                               None),
         )
         attrs['rules'] = [sg_rule]
         attrs.update(**kwargs)
@@ -171,3 +197,21 @@ class SecurityGroupServerAPIShimTestCase(base.BaseTestCase):
         self.rcache.record_resource_delete(self.ctx, 'Port', p1.id)
         self.sg_agent.security_groups_member_updated.assert_called_with(
             {s1.id})
+
+    def test_get_secgroup_ids_for_address_group(self):
+        ag = self._make_address_group_ovo()
+        sg1 = self._make_security_group_ovo(remote_address_group_id=ag.id)
+        sg2 = self._make_security_group_ovo(remote_address_group_id=ag.id)
+        sg3 = self._make_security_group_ovo()
+        sec_group_ids = self.shim.get_secgroup_ids_for_address_group(ag.id)
+        self.assertEqual(set([sg1.id, sg2.id]), set(sec_group_ids))
+        self.assertEqual(2, len(sec_group_ids))
+        self.assertNotIn(sg3.id, sec_group_ids)
+
+    def test_address_group_update_events(self):
+        ag = self._make_address_group_ovo()
+        self.sg_agent.address_group_updated.assert_called_with(ag.id)
+        self.sg_agent.address_group_updated.reset_mock()
+        self.rcache.record_resource_delete(self.ctx, resources.ADDRESSGROUP,
+            ag.id)
+        self.sg_agent.address_group_deleted.assert_called_with(ag.id)
