@@ -11,10 +11,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.api.definitions import l3 as l3_apidef
+from neutron_lib.api.definitions import router_availability_zone
+from neutron_lib import constants as lib_const
 from neutron_lib.plugins import constants
 
 from neutron.db.availability_zone import router as router_az_db
-from neutron.db import common_db_mixin
 from neutron.db import l3_agentschedulers_db
 from neutron.db import l3_db
 from neutron.extensions import l3
@@ -29,12 +31,12 @@ class AZL3ExtensionManager(test_az.AZExtensionManager):
                 l3.L3.get_resources())
 
 
-class AZRouterTestPlugin(common_db_mixin.CommonDbMixin,
-                         l3_db.L3_NAT_db_mixin,
+class AZRouterTestPlugin(l3_db.L3_NAT_db_mixin,
                          router_az_db.RouterAvailabilityZoneMixin,
                          l3_agentschedulers_db.AZL3AgentSchedulerDbMixin):
-    supported_extension_aliases = ["router", "l3_agent_scheduler",
-                                   "router_availability_zone"]
+    supported_extension_aliases = [l3_apidef.ALIAS,
+                                   lib_const.L3_AGENT_SCHEDULER_EXT_ALIAS,
+                                   router_availability_zone.ALIAS]
 
     @classmethod
     def get_plugin_type(cls):
@@ -61,7 +63,7 @@ class TestAZRouterCase(test_az.AZTestCommon, test_l3.L3NatTestCaseMixin):
         az_hints = ['nova2']
         with self.router(availability_zone_hints=az_hints) as router:
             res = self._show('routers', router['router']['id'])
-            self.assertItemsEqual(az_hints,
+            self.assertCountEqual(az_hints,
                                   res['router']['availability_zone_hints'])
 
     def test_create_router_with_azs(self):
@@ -69,7 +71,7 @@ class TestAZRouterCase(test_az.AZTestCommon, test_l3.L3NatTestCaseMixin):
         az_hints = ['nova2', 'nova3']
         with self.router(availability_zone_hints=az_hints) as router:
             res = self._show('routers', router['router']['id'])
-            self.assertItemsEqual(az_hints,
+            self.assertCountEqual(az_hints,
                                   res['router']['availability_zone_hints'])
 
     def test_create_router_without_az(self):
