@@ -10,7 +10,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 import netaddr
 from neutron_lib import constants
 from neutron_lib.tests import tools
@@ -52,6 +53,17 @@ class PortBindingDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
                                   BasePortBindingDbObjectTestCase):
     _test_class = ports.PortBinding
 
+    def test_get_duplicated_port_bindings(self):
+        port_id = self._create_test_port_id()
+        self.update_obj_fields({'port_id': port_id},
+                               objs=[self.objs[0], self.objs[1]])
+        for i in range(3):
+            _obj = self._make_object(self.obj_fields[i])
+            _obj.create()
+        dup_pb = ports.PortBinding.get_duplicated_port_bindings(self.context)
+        self.assertEqual(1, len(dup_pb))
+        self.assertEqual(port_id, dup_pb[0].port_id)
+
 
 class DistributedPortBindingIfaceObjTestCase(
         obj_test_base.BaseObjectIfaceTestCase):
@@ -378,12 +390,6 @@ class PortDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
         self.skipTest(
             'Port object loads segment info without relationships')
 
-    def test_v1_1_to_v1_0_drops_data_plane_status(self):
-        port_new = self._create_test_port()
-        port_v1_0 = port_new.obj_to_primitive(target_version='1.0')
-        self.assertNotIn('data_plane_status',
-                         port_v1_0['versioned_object.data'])
-
     def test_v1_2_to_v1_1_drops_segment_id_in_binding_levels(self):
         port_new = self._create_test_port()
         segment = network.NetworkSegment(
@@ -487,6 +493,18 @@ class PortDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
         self.assertNotIn('qos_network_policy_id',
                          port_v1_4['versioned_object.data'])
 
+    def test_v1_6_to_v1_5_drops_numa_affinity_policy(self):
+        port_new = self._create_test_port()
+        port_v1_5 = port_new.obj_to_primitive(target_version='1.5')
+        self.assertNotIn('numa_affinity_policy',
+                         port_v1_5['versioned_object.data'])
+
+    def test_v1_7_to_v1_6_drops_device_profile(self):
+        port_new = self._create_test_port()
+        port_v1_6 = port_new.obj_to_primitive(target_version='1.6')
+        self.assertNotIn('device_profile',
+                         port_v1_6['versioned_object.data'])
+
     def test_get_ports_ids_by_security_groups_except_router(self):
         sg_id = self._create_test_security_group_id()
         filter_owner = constants.ROUTER_INTERFACE_OWNERS_SNAT
