@@ -13,14 +13,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-
 import os
-
-import mock
+from unittest import mock
 
 from neutron.plugins.ml2.drivers.mech_sriov.agent.common \
     import exceptions as exc
 from neutron.plugins.ml2.drivers.mech_sriov.agent import eswitch_manager as esm
+from neutron.plugins.ml2.drivers.mech_sriov.agent import pci_lib
 from neutron.tests import base
 
 
@@ -215,7 +214,7 @@ class TestESwitchManagerApi(base.BaseTestCase):
                                                  self.PCI_SLOT, 1000)
             get_pci_mock.assert_called_once_with(self.PCI_SLOT)
             set_device_rate_mock.assert_called_once_with(
-                self.PCI_SLOT, self.MAX_RATE, 1000)
+                self.PCI_SLOT, {self.MAX_RATE: 1000})
 
     def test_set_device_min_tx_rate(self):
         with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
@@ -228,7 +227,7 @@ class TestESwitchManagerApi(base.BaseTestCase):
                                                     self.PCI_SLOT, 1000)
             get_pci_mock.assert_called_once_with(self.PCI_SLOT)
             set_device_rate_mock.assert_called_once_with(
-                self.PCI_SLOT, self.MIN_RATE, 1000)
+                self.PCI_SLOT, {self.MIN_RATE: 1000})
 
     def test_set_device_status_mismatch(self):
         with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
@@ -335,7 +334,8 @@ class TestESwitchManagerApi(base.BaseTestCase):
                            return_value=True):
             self.eswitch_mgr._clear_rate(pci_slot, rate_type)
             if passed:
-                set_rate_mock.assert_called_once_with(pci_slot, rate_type, 0)
+                set_rate_mock.assert_called_once_with(pci_slot,
+                                                      {rate_type: 0})
             else:
                 self.assertFalse(set_rate_mock.called)
 
@@ -430,6 +430,11 @@ class TestEmbSwitch(base.BaseTestCase):
                         "eswitch_manager.PciOsWrapper.scan_vf_devices",
                         return_value=self.SCANNED_DEVICES):
             self.emb_switch = esm.EmbSwitch(self.DEV_NAME, exclude_devices)
+        self.mock_get_vfs = mock.patch.object(esm.EmbSwitch,
+                                              '_get_vfs').start()
+        self.vf_rates = [{esm.IP_LINK_CAPABILITY_RATE: 500,
+                          esm.IP_LINK_CAPABILITY_MIN_TX_RATE: 250}]
+        self.mock_get_vfs.return_value = self.vf_rates
 
     @mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
                 "eswitch_manager.PciOsWrapper.scan_vf_devices",
@@ -521,61 +526,64 @@ class TestEmbSwitch(base.BaseTestCase):
                               self.emb_switch.set_device_spoofcheck,
                               self.WRONG_PCI_SLOT, True)
 
-    def test_set_device_rate_ok(self):
-        with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
-                        "PciDeviceIPWrapper.set_vf_rate") as pci_lib_mock:
-            self.emb_switch.set_device_rate(
-                self.PCI_SLOT, esm.IP_LINK_CAPABILITY_RATE, 2000)
-            pci_lib_mock.assert_called_with(
-                0, esm.IP_LINK_CAPABILITY_RATE, 2)
+    @mock.patch.object(pci_lib.PciDeviceIPWrapper, 'set_vf_rate')
+    def test_set_device_rate_ok(self, mock_set_vf_rate):
+        self.emb_switch.set_device_rate(
+            self.PCI_SLOT, {esm.IP_LINK_CAPABILITY_RATE: 2000})
+        self.vf_rates[0][esm.IP_LINK_CAPABILITY_RATE] = 2
+        mock_set_vf_rate.assert_called_with(0, self.vf_rates[0])
 
-    def test_set_device_max_rate_ok2(self):
-        with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
-                        "PciDeviceIPWrapper.set_vf_rate") as pci_lib_mock:
-            self.emb_switch.set_device_rate(
-                self.PCI_SLOT, esm.IP_LINK_CAPABILITY_RATE, 99)
-            pci_lib_mock.assert_called_with(
-                0, esm.IP_LINK_CAPABILITY_RATE, 1)
+        # No 'min_tx_rate' support
+        vf_rates = [{esm.IP_LINK_CAPABILITY_RATE: 500,
+                     esm.IP_LINK_CAPABILITY_MIN_TX_RATE: None}]
+        self.mock_get_vfs.return_value = vf_rates
+        self.emb_switch.set_device_rate(
+            self.PCI_SLOT, {esm.IP_LINK_CAPABILITY_RATE: 2000})
+        vf_rates[0] = {esm.IP_LINK_CAPABILITY_RATE: 2}
+        mock_set_vf_rate.assert_called_with(0, vf_rates[0])
 
-    def test_set_device_max_rate_rounded_ok(self):
-        with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
-                        "PciDeviceIPWrapper.set_vf_rate") as pci_lib_mock:
-            self.emb_switch.set_device_rate(
-                self.PCI_SLOT, esm.IP_LINK_CAPABILITY_RATE, 2001)
-            pci_lib_mock.assert_called_with(0, esm.IP_LINK_CAPABILITY_RATE, 2)
+    @mock.patch.object(pci_lib.PciDeviceIPWrapper, 'set_vf_rate')
+    def test_set_device_max_rate_ok2(self, mock_set_vf_rate):
+        self.emb_switch.set_device_rate(
+            self.PCI_SLOT, {esm.IP_LINK_CAPABILITY_RATE: 99})
+        self.vf_rates[0][esm.IP_LINK_CAPABILITY_RATE] = 1
+        mock_set_vf_rate.assert_called_with(0, self.vf_rates[0])
 
-    def test_set_device_max_rate_rounded_ok2(self):
-        with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
-                        "PciDeviceIPWrapper.set_vf_rate") as pci_lib_mock:
-            self.emb_switch.set_device_rate(
-                self.PCI_SLOT, esm.IP_LINK_CAPABILITY_RATE, 2499)
-            pci_lib_mock.assert_called_with(
-                0, esm.IP_LINK_CAPABILITY_RATE, 2)
+    @mock.patch.object(pci_lib.PciDeviceIPWrapper, 'set_vf_rate')
+    def test_set_device_max_rate_rounded_ok(self, mock_set_vf_rate):
+        self.emb_switch.set_device_rate(
+            self.PCI_SLOT, {esm.IP_LINK_CAPABILITY_RATE: 2001})
+        self.vf_rates[0][esm.IP_LINK_CAPABILITY_RATE] = 2
+        mock_set_vf_rate.assert_called_with(0, self.vf_rates[0])
 
-    def test_set_device_max_rate_rounded_ok3(self):
-        with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
-                        "PciDeviceIPWrapper.set_vf_rate") as pci_lib_mock:
-            self.emb_switch.set_device_rate(
-                self.PCI_SLOT, esm.IP_LINK_CAPABILITY_RATE, 2500)
-            pci_lib_mock.assert_called_with(
-                0, esm.IP_LINK_CAPABILITY_RATE, 3)
+    @mock.patch.object(pci_lib.PciDeviceIPWrapper, 'set_vf_rate')
+    def test_set_device_max_rate_rounded_ok2(self, mock_set_vf_rate):
+        self.emb_switch.set_device_rate(
+            self.PCI_SLOT, {esm.IP_LINK_CAPABILITY_RATE: 2499})
+        self.vf_rates[0][esm.IP_LINK_CAPABILITY_RATE] = 2
+        mock_set_vf_rate.assert_called_with(0, self.vf_rates[0])
 
-    def test_set_device_max_rate_disable(self):
-        with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
-                        "PciDeviceIPWrapper.set_vf_rate") as pci_lib_mock:
-            self.emb_switch.set_device_rate(
-                self.PCI_SLOT, esm.IP_LINK_CAPABILITY_RATE, 0)
-            pci_lib_mock.assert_called_with(
-                0, esm.IP_LINK_CAPABILITY_RATE, 0)
+    @mock.patch.object(pci_lib.PciDeviceIPWrapper, 'set_vf_rate')
+    def test_set_device_max_rate_rounded_ok3(self, mock_set_vf_rate):
+        self.emb_switch.set_device_rate(
+            self.PCI_SLOT, {esm.IP_LINK_CAPABILITY_RATE: 2500})
+        self.vf_rates[0][esm.IP_LINK_CAPABILITY_RATE] = 3
+        mock_set_vf_rate.assert_called_with(0, self.vf_rates[0])
 
-    def test_set_device_max_rate_fail(self):
-        with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
-                        "PciDeviceIPWrapper.set_vf_rate"):
-            self.assertRaises(
-                exc.InvalidPciSlotError,
-                self.emb_switch.set_device_rate,
-                self.WRONG_PCI_SLOT,
-                esm.IP_LINK_CAPABILITY_RATE, 1000)
+    @mock.patch.object(pci_lib.PciDeviceIPWrapper, 'set_vf_rate')
+    def test_set_device_max_rate_disable(self, mock_set_vf_rate):
+        self.emb_switch.set_device_rate(
+            self.PCI_SLOT, {esm.IP_LINK_CAPABILITY_RATE: 0})
+        self.vf_rates[0][esm.IP_LINK_CAPABILITY_RATE] = 0
+        mock_set_vf_rate.assert_called_with(0, self.vf_rates[0])
+
+    @mock.patch.object(pci_lib.PciDeviceIPWrapper, 'set_vf_rate')
+    def test_set_device_max_rate_fail(self, *args):
+        self.assertRaises(
+            exc.InvalidPciSlotError,
+            self.emb_switch.set_device_rate,
+            self.WRONG_PCI_SLOT,
+            {esm.IP_LINK_CAPABILITY_RATE: 1000})
 
     def test_get_pci_device(self):
         with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
@@ -741,15 +749,14 @@ class TestPciOsWrapper(base.BaseTestCase):
             self.assertTrue(esm.PciOsWrapper.pf_device_exists('p6p1'))
 
     def test_get_numvfs(self):
-        with mock.patch("six.moves.builtins.open",
+        with mock.patch("builtins.open",
                         mock.mock_open(read_data="63")) as mock_open:
             self.assertEqual(63, esm.PciOsWrapper.get_numvfs('dev1'))
             mock_open.assert_called_once_with(
                 esm.PciOsWrapper.NUMVFS_PATH % 'dev1')
 
     def test_get_numvfs_no_file(self):
-        with mock.patch("six.moves.builtins.open",
-                        side_effect=IOError()) as mock_open:
+        with mock.patch("builtins.open", side_effect=IOError()) as mock_open:
             self.assertEqual(-1, esm.PciOsWrapper.get_numvfs('dev1'))
             mock_open.assert_called_once_with(
                 esm.PciOsWrapper.NUMVFS_PATH % 'dev1')
