@@ -11,10 +11,11 @@
 #    under the License.
 
 import copy
+from unittest import mock
 
-import mock
 from neutron_lib import constants
 from neutron_lib import context
+from neutron_lib.services.qos import constants as qos_consts
 from oslo_utils import uuidutils
 
 from neutron.objects.qos import policy
@@ -40,7 +41,7 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
         self.context = context.get_admin_context()
         self.qos_driver = qos_driver.QosOVSAgentDriver()
         self.mock_clear_minimum_bandwidth_qos = mock.patch.object(
-            self.qos_driver, '_minimum_bandwidth_initialize').start()
+            self.qos_driver, '_qos_bandwidth_initialize').start()
         os_ken_app = mock.Mock()
         self.agent_api = ovs_ext_api.OVSAgentExtensionAPI(
                          ovs_bridge.OVSAgentBridge(
@@ -52,11 +53,12 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
         self.qos_driver.consume_api(self.agent_api)
         self.qos_driver.initialize()
         self.qos_driver.br_int = mock.Mock()
+        self.qos_driver.br_int.get_dp = mock.Mock(return_value=(mock.Mock(),
+                                                                mock.Mock(),
+                                                                mock.Mock()))
         self.qos_driver.br_int.get_egress_bw_limit_for_port = mock.Mock(
             return_value=(1000, 10))
         self.get_egress = self.qos_driver.br_int.get_egress_bw_limit_for_port
-        self.get_ingress = self.qos_driver.br_int.get_ingress_bw_limit_for_port
-        self.qos_driver.br_int.dump_flows_for = mock.Mock(return_value=None)
         self.qos_driver.br_int.del_egress_bw_limit_for_port = mock.Mock()
         self.delete_egress = (
             self.qos_driver.br_int.delete_egress_bw_limit_for_port)
@@ -119,8 +121,6 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
     def test_create_new_rules(self):
         self.qos_driver.br_int.get_egress_bw_limit_for_port = mock.Mock(
             return_value=(None, None))
-        self.qos_driver.br_int.get_ingress_bw_limit_for_port = mock.Mock(
-            return_value=(None, None))
         self.qos_driver.create(self.port, self.qos_policy)
         self.assertEqual(0, self.delete_egress.call_count)
         self.assertEqual(0, self.delete_ingress.call_count)
@@ -150,18 +150,12 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
         self.update_ingress.assert_not_called()
 
     def _test_delete_rules(self, qos_policy):
-        self.qos_driver.br_int.get_ingress_bw_limit_for_port = mock.Mock(
-            return_value=(self.rules[1].max_kbps,
-                          self.rules[1].max_burst_kbps))
         self.qos_driver.create(self.port, qos_policy)
         self.qos_driver.delete(self.port, qos_policy)
         self.delete_egress.assert_called_once_with(self.port_name)
         self.delete_ingress.assert_called_once_with(self.port_name)
 
     def _test_delete_rules_no_policy(self):
-        self.qos_driver.br_int.get_ingress_bw_limit_for_port = mock.Mock(
-            return_value=(self.rules[1].max_kbps,
-                          self.rules[1].max_burst_kbps))
         self.qos_driver.delete(self.port)
         self.delete_egress.assert_called_once_with(self.port_name)
         self.delete_ingress.assert_called_once_with(self.port_name)
@@ -188,14 +182,13 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
             self.rules[1].max_burst_kbps)
 
     def _assert_dscp_rule_create_updated(self):
-        # Assert add_flow is the last call
+        # Assert install_instructions is the last call
         self.assertEqual(
-            'add_flow',
+            'install_dscp_marking_rule',
             self.qos_driver.br_int.method_calls[-1][0])
 
-        self.qos_driver.br_int.add_flow.assert_called_once_with(
-            actions='mod_nw_tos:128,load:55->NXM_NX_REG2[0..5],resubmit(,0)',
-            in_port=mock.ANY, priority=65535, reg2=0, table=0)
+        self.qos_driver.br_int.install_dscp_marking_rule.\
+            assert_called_once_with(dscp_mark=mock.ANY, port=mock.ANY)
 
     def test_create_minimum_bandwidth(self):
         with mock.patch.object(self.qos_driver, 'update_minimum_bandwidth') \
@@ -208,9 +201,16 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
         with mock.patch.object(self.qos_driver.br_int,
                                'delete_minimum_bandwidth_queue') \
                 as mock_delete_minimum_bandwidth_queue:
-            self.qos_driver.delete_minimum_bandwidth({'port_id': 'port_id'})
-            mock_delete_minimum_bandwidth_queue.assert_called_once_with(
-                'port_id')
+            self.qos_driver.ports['p_id'] = {}
+            self.qos_driver.delete_minimum_bandwidth({'port_id': 'p_id'})
+            mock_delete_minimum_bandwidth_queue.assert_not_called()
+
+            mock_delete_minimum_bandwidth_queue.reset_mock()
+            self.qos_driver.ports['p_id'] = {
+                (qos_consts.RULE_TYPE_MINIMUM_BANDWIDTH,
+                 constants.EGRESS_DIRECTION): 'rule_port'}
+            self.qos_driver.delete_minimum_bandwidth({'port_id': 'p_id'})
+            mock_delete_minimum_bandwidth_queue.assert_called_once_with('p_id')
 
     def test_update_minimum_bandwidth_no_vif_port(self):
         with mock.patch.object(self.qos_driver.br_int,
