@@ -15,6 +15,7 @@
 from unittest import mock
 
 import netaddr
+from neutron_lib.api.definitions import portbindings as portbindings_api
 from neutron_lib.api.definitions import qos as qos_api
 from neutron_lib import constants
 from neutron_lib import context
@@ -74,7 +75,8 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         self.txn = _Context()
         mock_driver = mock.Mock()
         mock_driver._nb_idl.transaction.return_value = self.txn
-        self.qos_driver = qos_extension.OVNClientQosExtension(mock_driver)
+        self.qos_driver = qos_extension.OVNClientQosExtension(
+            driver=mock_driver)
         self._mock_rules = mock.patch.object(self.qos_driver,
                                              '_update_port_qos_rules')
         self.mock_rules = self._mock_rules.start()
@@ -187,6 +189,10 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
                          self.qos_driver._qos_rules(mock.ANY, mock.ANY))
 
     def _test__ovn_qos_rule_ingress(self, fip_id=None, ip_address=None):
+        if fip_id:
+            external_ids = {ovn_const.OVN_FIP_EXT_ID_KEY: fip_id}
+        else:
+            external_ids = {ovn_const.OVN_PORT_EXT_ID_KEY: 'port_id'}
         direction = constants.INGRESS_DIRECTION
         rule = {qos_constants.RULE_TYPE_BANDWIDTH_LIMIT: QOS_RULE_BW_1}
         match = self.qos_driver._ovn_qos_rule_match(
@@ -194,9 +200,8 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         expected = {'burst': 100, 'rate': 200, 'direction': 'to-lport',
                     'match': match,
                     'priority': qos_extension.OVN_QOS_DEFAULT_RULE_PRIORITY,
-                    'switch': 'neutron-network_id'}
-        if fip_id:
-            expected['external_ids'] = {ovn_const.OVN_FIP_EXT_ID_KEY: fip_id}
+                    'switch': 'neutron-network_id',
+                    'external_ids': external_ids}
         result = self.qos_driver._ovn_qos_rule(
             direction, rule, 'port_id', 'network_id', fip_id=fip_id,
             ip_address=ip_address, resident_port='resident_port')
@@ -209,15 +214,18 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         self._test__ovn_qos_rule_ingress(fip_id='fipid', ip_address='1.2.3.4')
 
     def _test__ovn_qos_rule_egress(self, fip_id=None, ip_address=None):
+        if fip_id:
+            external_ids = {ovn_const.OVN_FIP_EXT_ID_KEY: fip_id}
+        else:
+            external_ids = {ovn_const.OVN_PORT_EXT_ID_KEY: 'port_id'}
         direction = constants.EGRESS_DIRECTION
         rule = {qos_constants.RULE_TYPE_DSCP_MARKING: QOS_RULE_DSCP_1}
         match = self.qos_driver._ovn_qos_rule_match(
             direction, 'port_id', ip_address, 'resident_port')
         expected = {'direction': 'from-lport', 'match': match,
                     'dscp': 16, 'switch': 'neutron-network_id',
-                    'priority': qos_extension.OVN_QOS_DEFAULT_RULE_PRIORITY}
-        if fip_id:
-            expected['external_ids'] = {ovn_const.OVN_FIP_EXT_ID_KEY: fip_id}
+                    'priority': qos_extension.OVN_QOS_DEFAULT_RULE_PRIORITY,
+                    'external_ids': external_ids}
         result = self.qos_driver._ovn_qos_rule(
             direction, rule, 'port_id', 'network_id', fip_id=fip_id,
             ip_address=ip_address, resident_port='resident_port')
@@ -227,9 +235,8 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
                 qos_constants.RULE_TYPE_DSCP_MARKING: QOS_RULE_DSCP_2}
         expected = {'direction': 'from-lport', 'match': match,
                     'rate': 300, 'dscp': 20, 'switch': 'neutron-network_id',
-                    'priority': qos_extension.OVN_QOS_DEFAULT_RULE_PRIORITY}
-        if fip_id:
-            expected['external_ids'] = {ovn_const.OVN_FIP_EXT_ID_KEY: fip_id}
+                    'priority': qos_extension.OVN_QOS_DEFAULT_RULE_PRIORITY,
+                    'external_ids': external_ids}
         result = self.qos_driver._ovn_qos_rule(
             direction, rule, 'port_id', 'network_id', fip_id=fip_id,
             ip_address=ip_address, resident_port='resident_port')
@@ -244,28 +251,28 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
     def test__port_effective_qos_policy_id(self):
         port = {'qos_policy_id': 'qos1'}
         self.assertEqual(('qos1', 'port'),
-                         self.qos_driver._port_effective_qos_policy_id(port))
+                         self.qos_driver.port_effective_qos_policy_id(port))
 
         port = {'qos_network_policy_id': 'qos1'}
         self.assertEqual(('qos1', 'network'),
-                         self.qos_driver._port_effective_qos_policy_id(port))
+                         self.qos_driver.port_effective_qos_policy_id(port))
 
         port = {'qos_policy_id': 'qos_port',
                 'qos_network_policy_id': 'qos_network'}
         self.assertEqual(('qos_port', 'port'),
-                         self.qos_driver._port_effective_qos_policy_id(port))
+                         self.qos_driver.port_effective_qos_policy_id(port))
 
         port = {}
         self.assertEqual((None, None),
-                         self.qos_driver._port_effective_qos_policy_id(port))
+                         self.qos_driver.port_effective_qos_policy_id(port))
 
         port = {'qos_policy_id': None, 'qos_network_policy_id': None}
         self.assertEqual((None, None),
-                         self.qos_driver._port_effective_qos_policy_id(port))
+                         self.qos_driver.port_effective_qos_policy_id(port))
 
         port = {'qos_policy_id': 'qos1', 'device_owner': 'neutron:port'}
         self.assertEqual((None, None),
-                         self.qos_driver._port_effective_qos_policy_id(port))
+                         self.qos_driver.port_effective_qos_policy_id(port))
 
     def test_update_port(self):
         port = self.ports[0]
@@ -317,8 +324,13 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         # External port, OVN QoS extension does not apply.
         self.mock_rules.reset_mock()
         port.qos_policy_id = self.qos_policies[0].id
-        self.qos_driver.update_port(mock.ANY, port, original_port,
-                                    port_type=ovn_const.LSP_TYPE_EXTERNAL)
+        port_obj.PortBinding(self.ctx, port_id=port.id, host='host',
+                             profile={}, vif_type='',
+                             vnic_type=portbindings_api.VNIC_DIRECT).create()
+        # NOTE(ralonsoh): this OVO retrieval must include, in the port object,
+        # the port binding register created.
+        port = port_obj.Port.get_object(self.ctx, id=port.id)
+        self.qos_driver.update_port(mock.ANY, port, original_port)
         self.mock_rules.assert_not_called()
 
     def test_delete_port(self):
@@ -403,6 +415,37 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
             self.mock_rules.assert_has_calls(calls)
             self.mock_rules.reset_mock()
 
+    def test_update_network_external_ports(self):
+        """Test update network with external ports.
+
+        - port10: no QoS port policy
+        - port11: no QoS port policy but external
+        - port12: qos_policy0
+        """
+        policies_ports = [(self.qos_policies[0].id, {self.ports[0].id})]
+        self.ports[2].qos_policy_id = self.qos_policies[0].id
+        self.ports[2].update()
+        port_obj.PortBinding(self.ctx, port_id=self.ports[1].id, host='host',
+                             profile={}, vif_type='',
+                             vnic_type=portbindings_api.VNIC_DIRECT).create()
+        with mock.patch.object(self.qos_driver._driver._nb_idl,
+                               'get_lswitch_port') as mock_lsp:
+            mock_lsp.side_effect = [
+                mock.Mock(type=ovn_const.LSP_TYPE_LOCALNET),
+                mock.Mock(type=ovn_const.LSP_TYPE_EXTERNAL)]
+            for qos_policy_id, reference_ports in policies_ports:
+                self.networks[0].qos_policy_id = qos_policy_id
+                self.networks[0].update()
+                original_network = {'qos_policy_id': self.qos_policies[0]}
+                reviewed_port_ids = self.qos_driver.update_network(
+                    mock.ANY, self.networks[0], original_network, reset=True)
+                self.assertEqual(reference_ports, reviewed_port_ids)
+                calls = [mock.call(
+                    mock.ANY, self.ports[0].id, self.ports[0].network_id,
+                    qos_policy_id, None)]
+                self.mock_rules.assert_has_calls(calls)
+                self.mock_rules.reset_mock()
+
     def test_update_policy(self):
         """Test update QoS policy, networks and ports bound are updated.
 
@@ -453,6 +496,11 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         mock_update_fip.assert_called_once_with(self.txn, fip)
 
     def test_update_floatingip(self):
+        # NOTE(ralonsoh): this rule will always apply:
+        # - If the FIP is being deleted, "qos_del_ext_ids" is called;
+        #   "qos_add" and "qos_del" won't.
+        # - If the FIP is added or updated, "qos_del_ext_ids" won't be called
+        #   and "qos_add" or "qos_del" will, depending on the rule directions.
         nb_idl = self.qos_driver._driver._nb_idl
         fip = self.fips[0]
         original_fip = self.fips[1]
@@ -462,6 +510,7 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         self.qos_driver.update_floatingip(txn, fip)
         nb_idl.qos_del_ext_ids.assert_called_once()
         nb_idl.qos_add.assert_not_called()
+        nb_idl.qos_del.assert_not_called()
         nb_idl.reset_mock()
 
         # Attach a port and a router, not QoS policy
@@ -471,14 +520,19 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         self.qos_driver.update_floatingip(txn, fip)
         nb_idl.qos_del_ext_ids.assert_called_once()
         nb_idl.qos_add.assert_not_called()
+        nb_idl.qos_del.assert_not_called()
         nb_idl.reset_mock()
 
         # Add a QoS policy
         fip.qos_policy_id = self.qos_policies[0].id
         fip.update()
         self.qos_driver.update_floatingip(txn, fip)
-        nb_idl.qos_del_ext_ids.assert_called_once()
+        nb_idl.qos_del_ext_ids.assert_not_called()
+        # QoS DSCP rule has only egress direction, ingress one is deleted.
+        # Check "OVNClientQosExtension.update_floatingip" and how the OVN QoS
+        # rules are added (if there is a rule in this direction) or deleted.
         nb_idl.qos_add.assert_called_once()
+        nb_idl.qos_del.assert_called_once()
         nb_idl.reset_mock()
 
         # Remove QoS
@@ -489,6 +543,7 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         self.qos_driver.update_floatingip(txn, fip)
         nb_idl.qos_del_ext_ids.assert_called_once()
         nb_idl.qos_add.assert_not_called()
+        nb_idl.qos_del.assert_not_called()
         nb_idl.reset_mock()
 
         # Add again another QoS policy
@@ -497,8 +552,9 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         original_fip.qos_policy_id = None
         original_fip.update()
         self.qos_driver.update_floatingip(txn, fip)
-        nb_idl.qos_del_ext_ids.assert_called_once()
+        nb_idl.qos_del_ext_ids.assert_not_called()
         nb_idl.qos_add.assert_called_once()
+        nb_idl.qos_del.assert_called_once()
         nb_idl.reset_mock()
 
         # Detach the port and the router
@@ -512,6 +568,7 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         self.qos_driver.update_floatingip(txn, fip)
         nb_idl.qos_del_ext_ids.assert_called_once()
         nb_idl.qos_add.assert_not_called()
+        nb_idl.qos_del.assert_not_called()
         nb_idl.reset_mock()
 
         # Force reset (delete any QoS)
@@ -520,3 +577,4 @@ class TestOVNClientQosExtension(test_plugin.Ml2PluginV2TestCase):
         self.qos_driver.update_floatingip(txn, fip_dict)
         nb_idl.qos_del_ext_ids.assert_called_once()
         nb_idl.qos_add.assert_not_called()
+        nb_idl.qos_del.assert_not_called()
