@@ -13,7 +13,6 @@
 #    under the License.
 #
 
-import netaddr
 from neutron_lib import context as nctx
 from neutron_lib.db import api as db_api
 from neutron_lib.plugins import constants
@@ -67,7 +66,7 @@ class TestRevisionPlugin(test_plugin.Ml2PluginV2TestCase):
     def test_handle_expired_object(self):
         rp = directory.get_plugin('revision_plugin')
         with self.port():
-            with self.ctx.session.begin():
+            with db_api.CONTEXT_WRITER.using(self.ctx):
                 ipal_objs = port_obj.IPAllocation.get_objects(self.ctx)
                 if not ipal_objs:
                     raise Exception("No IP allocations available.")
@@ -156,18 +155,29 @@ class TestRevisionPlugin(test_plugin.Ml2PluginV2TestCase):
                 raise db_exc.DBDeadlock()
             db_api.sqla_listen(se.Session, 'before_commit',
                                concurrent_increment)
+
+            # Despite the revision number is bumped twice during the session
+            # transaction, the revision number is tested only once the first
+            # time the revision number service is executed for this session and
+            # object.
             self._update('ports', port['port']['id'], new,
                          headers={'If-Match': 'revision_number=%s' % rev},
+                         expected_code=exc.HTTPOk.code)
+            self._update('ports', port['port']['id'], new,
+                         headers={'If-Match': 'revision_number=%s' %
+                                              str(int(rev) + 2)},
+                         expected_code=exc.HTTPOk.code)
+            self._update('ports', port['port']['id'], new,
+                         headers={'If-Match': 'revision_number=1'},
                          expected_code=exc.HTTPPreconditionFailed.code)
 
     def test_port_ip_update_revises(self):
-        with self.port() as port:
+        with self.subnet() as subnet, self.port(subnet=subnet) as port:
             rev = port['port']['revision_number']
             new = {'port': {'fixed_ips': port['port']['fixed_ips']}}
             # ensure adding an IP allocation updates the port
-            next_ip = str(netaddr.IPAddress(
-                  new['port']['fixed_ips'][0]['ip_address']) + 1)
-            new['port']['fixed_ips'].append({'ip_address': next_ip})
+            free_ip = self._find_ip_address(subnet['subnet'])
+            new['port']['fixed_ips'].append({'ip_address': free_ip})
             response = self._update('ports', port['port']['id'], new)
             self.assertEqual(2, len(response['port']['fixed_ips']))
             new_rev = response['port']['revision_number']
@@ -180,6 +190,22 @@ class TestRevisionPlugin(test_plugin.Ml2PluginV2TestCase):
             new_rev = response['port']['revision_number']
             self.assertGreater(new_rev, rev)
 
+    def test_network_description_bumps_revision(self):
+        with self.network() as net:
+            rev = net['network']['revision_number']
+            data = {'network': {'description': 'Test Description'}}
+            response = self._update('networks', net['network']['id'], data)
+            new_rev = response['network']['revision_number']
+            self.assertEqual(rev + 1, new_rev)
+
+    def test_subnet_description_bumps_revision(self):
+        with self.subnet() as subnet:
+            rev = subnet['subnet']['revision_number']
+            data = {'subnet': {'description': 'Test Description'}}
+            response = self._update('subnets', subnet['subnet']['id'], data)
+            new_rev = response['subnet']['revision_number']
+            self.assertEqual(rev + 1, new_rev)
+
     def test_security_group_rule_ops_bump_security_group(self):
         s = {'security_group': {'tenant_id': 'some_tenant', 'name': '',
                                 'description': 's'}}
@@ -194,6 +220,7 @@ class TestRevisionPlugin(test_plugin.Ml2PluginV2TestCase):
                                      'remote_ip_prefix': '0.0.0.0/0',
                                      'ethertype': 'IPv4',
                                      'remote_group_id': None,
+                                     'remote_address_group_id': None,
                                      'direction': 'ingress',
                                      'security_group_id': sg['id']}}
         rule = self.cp.create_security_group_rule(self.ctx, r)
